import "../../core/dataset/AbstractStorableDataSet";
import "../../core/dataset/DataSetRow";
import "../../java/util/ArrayList";
import "../../core/dataset/StorableDataSetRow";
import "../../java/util/HashMap";
import "../../core/database/bean/ExecuteBean";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.base.dataset.DBStorableDataSet",
{
	extend : xlsys.core.dataset.AbstractStorableDataSet,
	
	events : 
	{
		// async supportted
		"saveCommit" : "qx.event.type.Event"
	},
	
	construct : function(dataBase, selectBean, tableName)
	{
		this.base(arguments);
		this.dataBase = dataBase;
		this.selectBean = selectBean;
		this.tableName = tableName;
	},
	
	members : 
	{
		dataBase : null,
		tableName : null,
		selectBean : null,
		
		getSerializableMembers : function()
		{
			var arr = this.base(arguments);
			arr.push("tableName");
			arr.push("selectBean");
			return arr;
		},

		open : async function()
		{
			if(!this.isEventLock("beforeOpen"))
			{
				this.fireEvent("beforeOpen");
				await this.fireAsyncEvent("beforeOpen");
			}
			try
			{
				var dataSet = await this.dataBase.sqlSelect(this.selectBean);
				this.setLockChange(true);
				this.rows.clear();
				this.cols.clear();
				this.colMap.clear();
				this.rowCursor = -1;
				this.colCursor = -1;
				// 添加列
				var colArr = dataSet.getCols().toArray();
				for(var i in colArr)
				{
					this.addCol(colArr[i]);
				}
				this._buildColMap();
				// 添加行
				var rowArr = dataSet.getRows().toArray();
				for(var i in rowArr)
				{
					this.createRow().setCells(rowArr[i].getCells());
				}
				this.gotoCell(-1, -1);
				this.setLockChange(false);
				if(!this.isEventLock("afterOpen"))
				{
					this.fireEvent("afterOpen");
					await this.fireAsyncEvent("afterOpen");
				}
			}
			catch(err)
			{
				if(!this.isEventLock("openFinally"))
				{
					this.fireEvent("openFinally");
					await this.fireAsyncEvent("openFinally");
				}
				throw err;
			}
			if(!this.isEventLock("openFinally"))
			{
				this.fireEvent("openFinally");
				await this.fireAsyncEvent("openFinally");
			}
			return;
		},
		
		save : async function()
		{
			if(!this.isChanged()) return true;
			if(!this.isEventLock("beforeSave"))
			{
				this.fireEvent("beforeSave");
				await this.fireAsyncEvent("beforeSave", true);
			}
			var success = false;
			var oldAutoCommit = true;
			try
			{
				oldAutoCommit = this.dataBase.getAutoCommit();
				// 启动事务处理
				if(this.isSaveTransaction()) await this.dataBase.setAutoCommit(false);
				var sqlBeanList = await this._prepareSqlBeanList();
				if(await this.dataBase.sqlExecute(sqlBeanList))
				{
					success = true;
					if(!this.isEventLock("afterSave"))
					{
						this.fireEvent("afterSave");
						await this.fireAsyncEvent("afterSave");
					}
				}
				if(this.saveTransaction&&!this.dataBase.getAutoCommit()) await this.dataBase.commit();
				if(success)
				{
					this.setChanged(false);
					if(this.toBeDeleteRows!=null) this.toBeDeleteRows.clear();
					if(!this.isLockChange())
					{
						var rowArr = this.getRows().toArray();
						for(var i in rowArr)
						{
							rowArr[i].setStatus(xlsys.core.dataset.DataSetRow.STATUS_COMMON);
						}
					}
				}
				if(!this.isEventLock("saveCommit"))
				{
					this.fireEvent("saveCommit");
					await this.fireAsyncEvent("saveCommit");
				}
			}
			catch(e)
			{
				if(this.saveTransaction&&!this.dataBase.getAutoCommit()) await this.dataBase.rollback();
				if(this.saveTransaction) await this.dataBase.setAutoCommit(oldAutoCommit);
				if(!this.isEventLock("saveFinally"))
				{
					this.fireEvent("saveFinally");
					await this.fireAsyncEvent("saveFinally");
				}
				throw e;
			}
			if(this.saveTransaction) await this.dataBase.setAutoCommit(oldAutoCommit);
			if(!this.isEventLock("saveFinally"))
			{
				this.fireEvent("saveFinally");
				await this.fireAsyncEvent("saveFinally");
			}
			return success;
		},
		
		_prepareSqlBeanList : async function()
		{
			var colArr = this.getCols().toArray();
			var sqlBeanList = new xlsys.java.util.ArrayList();
			var insertEb = null;
			var updateEb = null;
			var deleteEb = null;
			var rowArr = this.getRows().toArray();
			for(var i in rowArr)
			{
				var row = rowArr[i];
				if(!row.isChanged()) continue;
				if(row.getStatus()==xlsys.core.dataset.StorableDataSetRow.STATUS_REPLACE)
				{
					var updateDataMap = new xlsys.java.util.HashMap();
					var oldDataMap = new xlsys.java.util.HashMap();
					for(var j in colArr)
					{
						var dsc = colArr[j];
						var colName = dsc.getColumnName();
						if(this.noneedSaveCols==null||!this.noneedSaveCols.contains(colName)) updateDataMap.put(colName, this.getValue(row, colName));
						if(dsc.isPrimaryKey())
						{
							var oldValue = row.getCell(this.getColAt(colName)).getOldContent();
							if(oldValue==null) oldValue = row.getCell(this.getColAt(colName)).getContent();
							oldDataMap.put(colName, oldValue);
						}
					}
					if(updateEb==null) updateEb = new xlsys.core.database.bean.ExecuteBean(xlsys.core.database.bean.ExecuteBean.EXECUTE_TYPE_UPDATE, this.tableName);
					if(!updateDataMap.isEmpty()) updateEb.addData(updateDataMap, oldDataMap);
				}
				else
				{
					var dataMap = new xlsys.java.util.HashMap();
					for(var j in colArr)
					{
						var dsc = colArr[j];
						var colName = dsc.getColumnName();
						if(this.noneedSaveCols==null||!this.noneedSaveCols.contains(colName)) dataMap.put(colName, row.getCell(this.getColAt(colName)).getContent());
					}
					if(row.getStatus()==xlsys.core.dataset.StorableDataSetRow.STATUS_FOR_NEW)
					{
						if(insertEb==null) insertEb = new xlsys.core.database.bean.ExecuteBean(xlsys.core.database.bean.ExecuteBean.EXECUTE_TYPE_INSERT, this.tableName);
						if(!dataMap.isEmpty()) insertEb.addData(dataMap);
					}
					else if(row.getStatus()==xlsys.core.dataset.StorableDataSetRow.STATUS_UPDATE)
					{
						if(updateEb==null) updateEb = new xlsys.core.database.bean.ExecuteBean(xlsys.core.database.bean.ExecuteBean.EXECUTE_TYPE_UPDATE, this.tableName);
						if(!dataMap.isEmpty()) updateEb.addData(dataMap);
					}
				}
			}
			// 处理删除的列
			if(this.toBeDeleteRows!=null)
			{
				var tableInfo = await this.dataBase.getTableInfo(this.tableName, true);
				var toBeDeleteRowArr = this.toBeDeleteRows.toArray();
				var pkColArr = tableInfo.getPkColSet().toArray();
				for(var i in toBeDeleteRowArr)
				{
					var row = toBeDeleteRowArr[i];
					if(deleteEb==null) deleteEb = new xlsys.core.database.bean.ExecuteBean(xlsys.core.database.bean.ExecuteBean.EXECUTE_TYPE_DELETE, this.tableName);
					var dataMap = new xlsys.java.util.HashMap();
					for(var j in pkColArr)
					{
						var colName = pkColArr[j];
						dataMap.put(colName, row.getCell(this.getColAt(colName)).getContent());
					}
					deleteEb.addData(dataMap);
				}
			}
			if(insertEb!=null) sqlBeanList.add(insertEb);
			if(updateEb!=null) sqlBeanList.add(updateEb);
			if(deleteEb!=null) sqlBeanList.add(deleteEb);
			return sqlBeanList;
		},

		setRowStatusAfterSetValueSuccess : function(rowAt, colAt)
		{
			var isPkCol = this.getColumn(colAt).isPrimaryKey();
			var row = this.getRow(rowAt);
			if(row.getStatus()!=xlsys.core.dataset.StorableDataSetRow.STATUS_FOR_NEW)
			{
				if(isPkCol) row.setStatus(xlsys.core.dataset.StorableDataSetRow.STATUS_REPLACE);
				else if(row.getStatus()!=xlsys.core.dataset.StorableDataSetRow.STATUS_REPLACE) row.setStatus(xlsys.core.dataset.StorableDataSetRow.STATUS_UPDATE);
			}
		},
		
		getTableName : function()
		{
			return this.tableName;
		},

		getSelectBean : function()
		{
			return this.selectBean;
		},
		
		getSql : function()
		{
			return this.selectBean.getSql();
		}
	}
});