import "./DBStorableDataSet";
import "../../core/dataset/IVirtualizableDataSet";
import "../../core/dataset/DataSetRow";
import "../../core/util/Pair";
import "../../core/log/Log";
import "../../core/util/ObjectUtil";
import "../../java/util/HashMap";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.base.dataset.DBVSDataSet",
{
	extend : xlsys.base.dataset.DBStorableDataSet,
	implement : [xlsys.core.dataset.IVirtualizableDataSet],
	
	construct : function(dataBase, selectBean, tableName)
	{
		this.base(arguments, dataBase, selectBean, tableName);
		this.expandPerCount = 32;
	},
	
	members : 
	{
		expandPerCount : null,
		
		setRowCount : function(rowCount)
		{
			var curRowCount = this.getRowCount();
			if(curRowCount<rowCount)
			{
				this.setEventLock(true);
				this.setLockChange(true);
				var oldRowCursor = this.getRowCursor();
				for(var i=0;i<rowCount-curRowCount;++i)
				{
					var row = this.createRow();
					row.setStatus(xlsys.core.dataset.DataSetRow.STATUS_VIRTUAL);
				}
				this.gotoRow(oldRowCursor);
				this.setEventLock(false);
				this.setLockChange(false);
			}
		},

		getExpandPerCount : function()
		{
			return this.expandPerCount;
		},

		setExpandPerCount : function(expandPerCount)
		{
			this.expandPerCount = expandPerCount;
		},

		getPageRangeOfRowAt : function(wantRowAt)
		{
			this.checkRowRegion(wantRowAt);
			var beginRowIdx = Math.floor(wantRowAt / this.expandPerCount) * this.expandPerCount;
			var endRowIdx = Math.min(beginRowIdx + this.expandPerCount-1, this.getRowCount()-1);
			return new xlsys.core.util.Pair(beginRowIdx, endRowIdx);
		},
		
		expandToRow : async function(wantRowAt)
		{
			this.checkRowRegion(wantRowAt);
			if(!this.isVirtualRow(wantRowAt)) return true;
			var range = this.getPageRangeOfRowAt(wantRowAt);
			var success = false;
			try
			{
				success = await this._expandRow(range.first, range.second);
			}
			catch(e)
			{
				success = false;
				xlsys.core.log.Log.getInstance().printlnError(e);
			}
			return success;
		},
		
		_expandRow : async function(beginRowAt, endRowAt)
		{
			xlsys.core.log.Log.getInstance().printlnDebug("Expand row from " + beginRowAt + " to " + endRowAt);
			var dataSet = await this.dataBase.sqlSelect(this.selectBean, beginRowAt, endRowAt);
			var tempRows = dataSet.getRows();
			for(var i=0;i<tempRows.size();++i)
			{
				var row = this.getRows().get(i+beginRowAt);
				row.cloneOf(tempRows.get(i));
				row.setStatus(xlsys.core.dataset.DataSetRow.STATUS_COMMON);
			}
			return true;
		},
		
		expandAllRows : async function()
		{
			var rowCount = this.getRowCount();
			var allPromises = new Array();
			for(var i=0;i<rowCount;i+=this.expandPerCount)
			{
				allPromises.push(this.expandToRow(i));
			}
			await Promise.all(allPromises);
			return;
		},

		/**
		 * isVirtualRow(row);
		 * isVirtualRow(rowAt);
		 */
		isVirtualRow : function(rowOrRowAt)
		{
			var row = null;
			if(qx.lang.Type.isNumber(rowOrRowAt))
			{
				this.checkRowRegion(rowOrRowAt);
				row = this.getRow(rowOrRowAt);
			}
			else row = rowOrRowAt;
			return xlsys.core.util.ObjectUtil.hasBitProp(row.getStatus(), xlsys.core.dataset.DataSetRow.STATUS_VIRTUAL);
		},
		
		removeRow : function(rowOrRowAt)
		{
			throw new Error("Can not use removeRow, use asyncRemoveRow instead");
		},
		
		asyncRemoveRow : async function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			await this.expandToRow(rowAt);
			let row = this.rows.get(rowAt);
			if(!this.isEventLock("beforeRemoveRow"))
			{
				this.fireDataEvent("beforeRemoveRow", row);
				await this.fireAsyncDataEvent("beforeRemoveRow", row);
			}
			this.rows.removeAt(rowAt);
			if(!this.isEventLock("afterRemoveRow"))
			{
				this.fireDataEvent("afterRemoveRow", row);
				await this.fireAsyncDataEvent("afterRemoveRow", row);
			}
			return row;
		},

		getValue : function()
		{
			throw new Error("Can not use getValue, use asyncGetValue instead");
		},

		asyncGetValue : async function()
		{
			var rowAt = null;
			var colAt = null;
			if(arguments.length==0)
			{
				rowAt = this.rowCursor;
				colAt = this.colCursor;
			}
			else if(arguments.length==1)
			{
				rowAt = this.rowCursor;
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
			}
			else
			{
				if(qx.lang.Type.isNumber(arguments[0])) rowAt = arguments[0];
				else rowAt = this.getRowAt(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colAt = arguments[1];
				else colAt = this.getColAt(arguments[1]);
			}
			this.checkRowRegion(rowAt);
			this.checkColRegion(colAt);
			await this.expandToRow(rowAt);
			var row = this.rows.get(rowAt);
			return row.getCell(colAt).getContent();
		},
		
		setValue : function()
		{
			throw new Error("Can not use setValue, use asyncSetValue instead");
		},

		asyncSetValue : async function()
		{
			var rowAt = null;
			var colAt = null;
			var value = null;
			if(arguments.length==1)
			{
				rowAt = this.rowCursor;
				colAt = this.colCursor;
				value = arguments[0];
			}
			else if(arguments.length==2)
			{
				rowAt = this.rowCursor;
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
				value = arguments[1];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0])) rowAt = arguments[0];
				else rowAt = this.getRowAt(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colAt = arguments[1];
				else colAt = this.getColAt(arguments[1]);
				value = arguments[2];
			}
			this.checkRowRegion(rowAt);
			this.checkColRegion(colAt);
			await this.expandToRow(rowAt);
			var oldValue = this.rows.get(rowAt).getCell(colAt).getContent();
			if(oldValue==value||(oldValue!=null&&value!=null&&xlsys.core.util.ObjectUtil.hashcode(oldValue)==xlsys.core.util.ObjectUtil.hashcode(value))) return false;
			var dataMap = new xlsys.java.util.HashMap();
			dataMap.put("rowAt", rowAt);
			dataMap.put("colAt", colAt);
			dataMap.put("oldValue", oldValue);
			dataMap.put("value", value);
			if(!this.isEventLock("beforePostValue"))
			{
				this.fireDataEvent("beforePostValue", dataMap);
				await this.fireAsyncDataEvent("beforePostValue", dataMap);
			}
			this.rows.get(rowAt).getCell(colAt).setContent(value);
			if(!this.isEventLock("afterPostValue"))
			{
				this.fireDataEvent("afterPostValue", dataMap);
				await this.fireAsyncDataEvent("afterPostValue", dataMap);
			}
			return true;
		},
		
		open : async function()
		{
			if(!this.isEventLock("beforeOpen"))
			{
				this.fireEvent("beforeOpen");
				await this.fireAsyncEvent("beforeOpen");
			}
			try
			{
				var dataSet = await this.dataBase.sqlSelect(this.selectBean, 0, this.getExpandPerCount()-1);
				this.setLockChange(true);
				this.rows.clear();
				this.cols.clear();
				this.colMap.clear();
				this.rowCursor = -1;
				this.colCursor = -1;
				// 添加列
				var colArr = dataSet.getCols().toArray();
				for(var i in colArr)
				{
					this.addCol(colArr[i]);
				}
				this._buildColMap();
				// 添加行
				var rowArr = dataSet.getRows().toArray();
				for(var i in rowArr)
				{
					this.createRow().setCells(rowArr[i].getCells());
				}
				// 设置行数
				this.setRowCount(await this.dataBase.getResultCount(this.selectBean));
				this.setLockChange(false);
				if(!this.isEventLock("afterOpen"))
				{
					this.fireEvent("afterOpen");
					await this.fireAsyncEvent("afterOpen");
				}
			}
			catch(err)
			{
				if(!this.isEventLock("openFinally"))
				{
					this.fireEvent("openFinally");
					await this.fireAsyncEvent("openFinally");
				}
				throw err;
			}
			if(!this.isEventLock("openFinally"))
			{
				this.fireEvent("openFinally");
				await this.fireAsyncEvent("openFinally");
			}
			return;
		}
	}
});