import "../util/RequestUtil";
import "../../../base/Base";
import "../../../java/util/HashMap";
import "../util/SystemUtil";
import "../../../core/log/Log";
import "../../../core/util/IOUtil";
import "../io/ClientStorage";
var qx = require("xlsys-qx");

qx.Mixin.define("xlsys.client.base.buffer.MClientBaseBuffer", 
{
	statics : 
	{
		versionMap : null,
		accessBuffers : {}
	},
	
	members : 
	{
		_getBufferData : async function(envId, bufferName, paramMap)
		{
			if(!(await this.isBufferComplete(envId, bufferName))) await this.reloadDataFromLocalStorage(envId, bufferName);
			var ret = await this.doGetBufferData(envId, bufferName, paramMap);
			if(await this.isBufferComplete(envId, bufferName))
			{
				var storageObject = await this.getStorageObject(envId, bufferName);
				await this.saveDataToLocalStorage(envId, bufferName, storageObject);
			}
			return ret;
		},
		
		_updateCurrentVersion : async function(envId, bufferName)
		{
			var version = await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_UPDATE_BUFFER_VERSION, bufferName);
			var versionMap = xlsys.client.base.buffer.MClientBaseBuffer.versionMap;
			var tempMap = versionMap.get(envId);
			if(tempMap==null)
			{
				tempMap = new xlsys.java.util.HashMap();
				versionMap.put(envId, tempMap);
			}
			tempMap.put(bufferName, version);
			return version;
		},
		
		_getCurrentVersion : async function(envId, bufferName)
		{
			if(xlsys.client.base.buffer.MClientBaseBuffer.versionMap==null) xlsys.client.base.buffer.MClientBaseBuffer.versionMap = new xlsys.java.util.HashMap();
			var versionMap = xlsys.client.base.buffer.MClientBaseBuffer.versionMap;
			var tempMap = versionMap.get(envId);
			if(tempMap==null)
			{
				tempMap = new xlsys.java.util.HashMap();
				versionMap.put(envId, tempMap);
			}
			var version = tempMap.get(bufferName);
			if(version==-1) version = await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_GET_BUFFER_VERSION, bufferName);
			tempMap.put(bufferName, version);
			return version;
		},
		
		_reloadAllData : async function(envId, bufferName, forceLoad)
		{
			if(forceLoad==null) forceLoad = false;
			if(!(await this.reloadDataFromLocalStorage(envId, bufferName)))
			{
				await this.reloadDataDirectly(envId, bufferName, null, forceLoad);
				if(await this.isBufferComplete(envId, bufferName))
				{
					var storageObject = await this.getStorageObject(envId, bufferName);
					await this.saveDataToLocalStorage(envId, bufferName, storageObject);
				}
			}
			return;
		},
		
		_reloadData : async function(envId, bufferName, paramMap, forceLoad)
		{
			if(forceLoad==null) forceLoad = false;
			if(paramMap==null) await this.reloadAllData(envId, bufferName, forceLoad);
			else
			{
				await this.reloadDataDirectly(envId, bufferName, paramMap, forceLoad);
				if(await this.isBufferComplete(envId, bufferName))
				{
					var storageObject = await this.getStorageObject(envId, bufferName);
					await this.saveDataToLocalStorage(envId, bufferName, storageObject);
				}
			}
		},
		
		_saveDataToLocalStorage : async function(envId, bufferName, storageObject)
		{
			if(xlsys.client.base.util.SystemUtil.isDebug()) return false;
			var version = await this.getCurrentVersion(envId, bufferName);
			var localVersion = await this.getLocalStorageVersion(envId, bufferName);
			if(localVersion>=version) return true;
			if(xlsys.client.base.buffer.MClientBaseBuffer.accessBuffers[bufferName]) return false;
			xlsys.client.base.buffer.MClientBaseBuffer.accessBuffers[bufferName] = true;
			xlsys.core.log.Log.getInstance().printlnInfo("Prepare save buffer to local. EnvId : " + envId + "  BufferName : " + bufferName);
			var bufferData = xlsys.core.util.IOUtil.getCommonJSONFromObj(storageObject);
			let success = await xlsys.client.base.io.ClientStorage.getInstance().setItem(bufferName, ""+envId, bufferData);
			if(success)
			{
				var key = envId + "@@version";
				await xlsys.client.base.io.ClientStorage.getInstance().setItem(bufferName, key, version);
			}
			xlsys.core.log.Log.getInstance().printlnInfo("Save buffer to local "+(success?"success":"fail")+"! EnvId : " + envId + "  BufferName : " + bufferName);
			delete xlsys.client.base.buffer.MClientBaseBuffer.accessBuffers[bufferName];
			return success;
		},
		
		_reloadDataFromLocalStorage : async function(envId, bufferName)
		{
			if(xlsys.client.base.util.SystemUtil.isDebug()) return false;
			var curVersion = await this.getCurrentVersion(envId, bufferName);
			if(curVersion<0) return false;
			var localVersion = await this.getLocalStorageVersion(envId, bufferName);
			if(localVersion<0||curVersion>localVersion) return false;
			if(xlsys.client.base.buffer.MClientBaseBuffer.accessBuffers[bufferName]) return false;
			xlsys.client.base.buffer.MClientBaseBuffer.accessBuffers[bufferName] = true;
			// 从本地存储中加载缓冲
			xlsys.core.log.Log.getInstance().printlnInfo("Prepare load buffer from local. EnvId : " + envId + "  BufferName : " + bufferName);
			let bufferData = await xlsys.client.base.io.ClientStorage.getInstance().getItem(bufferName, ""+envId);
			let success = false;
			if(bufferData!=null)
			{
				var storageObject = xlsys.core.util.IOUtil.getObjFromCommonJSON(bufferData);
				success = await this.loadDataFromStorageObject(envId, bufferName, storageObject);
			}
			xlsys.core.log.Log.getInstance().printlnInfo("Load buffer from local " + (success?"success":"failed") + "! EnvId : " + envId + "  BufferName : " + bufferName);
			delete xlsys.client.base.buffer.MClientBaseBuffer.accessBuffers[bufferName];
			return success;
		},
		
		_getLocalStorageVersion : async function(envId, bufferName)
		{
			if(xlsys.client.base.util.SystemUtil.isDebug()) return -1;
			var key = envId + "@@version";
			var version = await xlsys.client.base.io.ClientStorage.getInstance().getItem(bufferName, key);
			if(version==null) return -1;
			return version;
		}
	}
});