import "./ClientBaseBuffer";
import "../../../java/util/HashMap";
import "../../../core/util/ObjectUtil";
import "../../../core/env/Environment";
import "../ClientBase";
import "../../../core/session/ISession";
import "../../../core/Core";
import "../../../core/database/IDataBase";
import "../database/ClientDataBase";
import "../../../java/util/ArrayList";
var qx = require("xlsys-qx");

/**
 * 模型缓冲抽象类
 */
qx.Class.define("xlsys.client.base.buffer.ModelBuffer", 
{
	type : "abstract",
	extend : xlsys.client.base.buffer.ClientBaseBuffer,

	statics : 
	{
		BUFFER_KEY_MODEL_KEY : "_BUFFER_KEY_MODEL_KEY",
		BUFFER_KEY_DATABASE : "_BUFFER_KEY_DATABASE",
		BUFFER_KEY_SESSION : "_BUFFER_KEY_SESSION",
		BUFFER_KEY_ENV_ID : "_BUFFER_KEY_ENV_ID"
	},
	
	construct : function()
	{
		this.base(arguments);
		this.modelBufferDelegateMap = new xlsys.java.util.HashMap();
		this.modelCountMap = new xlsys.java.util.HashMap();
	},
	
	members : 
	{
		modelBufferDelegateMap : null,
		modelCountMap : null,

		/**
		 * initAllModels(bufferName, dataBase);
		 * initAllModels(bufferName, dataBase, modelBufferDelegate);
		 */
		initAllModels : async function(bufferName, dataBase, modelBufferDelegate)
		{
			if(modelBufferDelegate==null)
			{
				modelBufferDelegate = this.modelBufferDelegateMap.get(bufferName);
				if(modelBufferDelegate==null) throw new Error("Please bind model buffer delegate first!");
			}
			var envId = dataBase.getEnvId();
			var modelMap = await this.getBufferMap(envId, bufferName);
			var allCount = await this.getModelCount(bufferName, dataBase);
			if(allCount==-1||modelMap.size()<allCount)
			{
				modelMap.clear();
				modelMap.putAll(await modelBufferDelegate.createAllModels(dataBase));
			}
		},
		
		/**
		 * getModelCount(bufferName);
		 * getModelCount(bufferName, envId);
		 * getModelCount(bufferName, session);
		 * getModelCount(bufferName, dataBase);
		 */
		getModelCount : async function(bufferName, sed)
		{
			var envId = null;
			var dataBase = null;
			if(sed==null) envId = xlsys.core.util.ObjectUtil.objectToInt(xlsys.core.env.Environment.getInstance().getString(xlsys.client.base.ClientBase.ENV_ENV_ID));
			else if(qx.lang.Type.isNumber(sed)) envId = sed;
			else if(qx.Class.implementsInterface(sed, xlsys.core.session.ISession)) envId = sed.getAttribute(xlsys.core.Core.SESSION_ENV_ID);
			else if(qx.Class.implementsInterface(sed, xlsys.core.database.IDataBase))
			{
				dataBase = sed;
				envId = dataBase.getEnvId();
			}
			var countMap = this.modelCountMap.get(envId);
			if(countMap==null)
			{
				countMap = new xlsys.java.util.HashMap();
				this.modelCountMap.put(envId, countMap);
			}
			var allCount = countMap.get(bufferName);
			if(allCount==null)
			{
				var modelBufferDelegate = this.modelBufferDelegateMap.get(bufferName);
				if(modelBufferDelegate==null) throw new Error("Please bind model buffer delegate first!");
				var needClose = false;
				if(dataBase==null)
				{
					dataBase = xlsys.client.base.database.ClientDataBase.getInstance();
					needClose = true;
				}
				allCount = await modelBufferDelegate.getAllCount(dataBase);
				if(needClose) await dataBase.close();
				countMap.put(bufferName, allCount);
			}
			return allCount;
		},
				
		bindModelBufferDelegate : function(bufferName, modelBufferDelegate)
		{
			this.modelBufferDelegateMap.put(bufferName, modelBufferDelegate);
		},
		
		/**
		 * getAllModels(bufferName);
		 * getAllModels(bufferName, session);
		 * getAllModels(bufferName, envId);
		 * getAllModels(bufferName, dataBase);
		 */
		getAllModels : async function(bufferName, sed)
		{
			var paramMap = new xlsys.java.util.HashMap();
			let envId = xlsys.core.util.ObjectUtil.objectToInt(xlsys.core.env.Environment.getInstance().getString(xlsys.client.base.ClientBase.ENV_ENV_ID));
			if(sed==null) sed = envId;
			else if(qx.lang.Type.isNumber(sed)) paramMap.put(this.self(arguments).BUFFER_KEY_ENV_ID, sed);
			else if(qx.Class.implementsInterface(sed, xlsys.core.session.ISession)) paramMap.put(this.self(arguments).BUFFER_KEY_SESSION, sed);
			else if(qx.Class.implementsInterface(sed, xlsys.core.database.IDataBase)) paramMap.put(this.self(arguments).BUFFER_KEY_DATABASE, sed);
			return await this.getBufferData(envId, bufferName, paramMap);
		},
		
		__getAllModelsWithSession : async function(bufferName, session, modelBufferDelegate)
		{
			var envId = session.getAttribute(xlsys.core.Core.SESSION_ENV_ID);
			return await this.__getAllModelsWithEnvId(bufferName, envId, modelBufferDelegate);
		},
				
		__getAllModelsWithEnvId : async function(bufferName, envId, modelBufferDelegate)
		{
			var modelMap = await this.getBufferMap(envId, bufferName);
			var modelList = null;
			var allCount = await this.getModelCount(bufferName, envId);
			if(allCount==-1||modelMap.size()<allCount)
			{
				var dataBase = xlsys.client.base.database.ClientDataBase.getInstance();
				await this.initAllModels(bufferName, dataBase, modelBufferDelegate);
				dataBase.close();
			}
			modelList = new xlsys.java.util.ArrayList();
			var valueArray = modelMap.values();
			for(var i=0;i<valueArray.length;++i)
			{
				modelList.add(valueArray[i]);
			}
			return modelList;
		},

		__getAllModelsWithDataBase : async function(bufferName, dataBase, modelBufferDelegate)
		{
			var modelMap = await this.getBufferMap(dataBase.getEnvId(), bufferName);
			var allCount = await this.getModelCount(bufferName, dataBase.getEnvId());
			if(allCount==-1||modelMap.size()<allCount) await this.initAllModels(bufferName, dataBase, modelBufferDelegate);
			var modelList = new xlsys.java.util.ArrayList();
			var valueArray = modelMap.values();
			for(var i=0;i<valueArray.length;++i)
			{
				modelList.add(valueArray[i]);
			}
			return modelList;
		},
		
		/**
		 * getModel(bufferName, session, key);
		 * getModel(bufferName, envId, key);
		 * getModel(bufferName, dataBase, key);
		 */
		getModel : async function(bufferName, sed, key, modelBufferDelegate)
		{
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(this.self(arguments).BUFFER_KEY_MODEL_KEY, key);
			let envId = xlsys.core.util.ObjectUtil.objectToInt(xlsys.core.env.Environment.getInstance().getString(xlsys.client.base.ClientBase.ENV_ENV_ID));
			if(sed==null) sed = envId;
			if(qx.lang.Type.isNumber(sed)) paramMap.put(this.self(arguments).BUFFER_KEY_ENV_ID, sed);
			else if(qx.Class.implementsInterface(sed, xlsys.core.session.ISession)) paramMap.put(this.self(arguments).BUFFER_KEY_SESSION, sed);
			else if(qx.Class.implementsInterface(sed, xlsys.core.database.IDataBase)) paramMap.put(this.self(arguments).BUFFER_KEY_DATABASE, sed);
			return await this.getBufferData(envId, bufferName, paramMap);
		},
				
		__getModelWithSession : async function(bufferName, session, key, modelBufferDelegate)
		{
			var envId = session.getAttribute(xlsys.core.Core.SESSION_ENV_ID);
			return await this.__getModelWithEnvId(bufferName, envId, key, modelBufferDelegate);
		},
				
		__getModelWithEnvId : async function(bufferName, envId, key, modelBufferDelegate)
		{
			if(key==null) return null;
			var modelMap = await this.getBufferMap(envId, bufferName);
			var model = modelMap.get(key);
			if(model==null)
			{
				var dataBase = xlsys.client.base.database.ClientDataBase.getInstance();
				model = await this.__getModelWithDataBase(bufferName, dataBase, key, modelBufferDelegate);
				dataBase.close();
			}
			return model;
		},

		__getModelWithDataBase : async function(bufferName, dataBase, key, modelBufferDelegate)
		{
			if(key==null) return null;
			var modelMap = await this.getBufferMap(dataBase.getEnvId(), bufferName);
			var model = modelMap.get(key);
			if(model==null)
			{
				model = await modelBufferDelegate.createModel(dataBase, key);
				modelMap.put(key, model);
			}
			return model;
		},
		
		getStorageObject : async function(envId, bufferName)
		{
			var bufferMap = this.allEnvBufferMap.get(envId).get(bufferName);
			var modelCount = this.modelCountMap.get(envId).get(bufferName);
			var storageObject = new Array();
			storageObject.push(bufferMap);
			storageObject.push(modelCount);
			return storageObject;
		},
		
		isBufferComplete : async function(envId, bufferName)
		{
			var envBufferMap = this.allEnvBufferMap.get(envId);
			var countMap = this.modelCountMap.get(envId);
			if(envBufferMap==null||countMap==null) return false;
			var bufferMap = envBufferMap.get(bufferName);
			var count = countMap.get(bufferName);
			if(bufferMap==null||count==null) return false;
			if(bufferMap.size()==count) return true;
			return false;
		},
		
		reloadDataDirectly : async function(envId, bufferName, paramMap, forceLoad)
		{
			if(forceLoad==null) forceLoad = false;
			var reloadAll = false;
			if(paramMap==null) reloadAll = true;
			else
			{
				var key = paramMap.get(this.self(arguments).BUFFER_KEY_MODEL_KEY);
				if(key==null) reloadAll = true;
				else
				{
					this.getBufferMap(envId, bufferName).remove(key);
					if(forceLoad) await this.getModel(bufferName, envId, key);
				}
			}
			if(reloadAll)
			{
				this.getBufferMap(envId, bufferName).clear();
				var countMap = this.modelCountMap.get(envId);
				if(countMap!=null) countMap.remove(bufferName);
				if(forceLoad) await this.getAllModels(bufferName, envId);
			}
		},
		
		loadDataFromStorageObject : async function(envId, bufferName, storageObj)
		{
			var objArr = storageObj;
			var bufferMap = objArr[0];
			var curBufferMap = this.getBufferMap(envId, bufferName);
			curBufferMap.putAll(bufferMap);
			var modelCount = objArr[1];
			var countMap = this.modelCountMap.get(envId);
			if(countMap==null)
			{
				countMap = this.modelCountMap.get(envId);
				if(countMap==null)
				{
					countMap = new xlsys.java.util.HashMap();
					this.modelCountMap.put(envId, countMap);
				}
			}
			countMap.put(bufferName, modelCount);
			return true;
		},
		
		doGetBufferData : async function(envId, bufferName, paramMap)
		{
			var ret = null;
			var modelBufferDelegate = this.modelBufferDelegateMap.get(bufferName);
			if(modelBufferDelegate==null) throw new Error("Please bind model buffer delegate first!");
			if(paramMap==null) ret = await this.dispatchGetAllModels(bufferName, null, envId, null, modelBufferDelegate);
			else
			{
				var key = paramMap.get(this.self(arguments).BUFFER_KEY_MODEL_KEY);
				var session = paramMap.get(this.self(arguments).BUFFER_KEY_SESSION);
				var dataBase = paramMap.get(this.self(arguments).BUFFER_KEY_DATABASE);
				var tempEnvId = paramMap.get(this.self(arguments).BUFFER_KEY_ENV_ID);
				if(tempEnvId==null) tempEnvId = envId;
				if(key==null) ret = await this.dispatchGetAllModels(bufferName, session, tempEnvId, dataBase, modelBufferDelegate);
				else ret = await this.dispatchGetModel(bufferName, session, tempEnvId, dataBase, key, modelBufferDelegate);
			}
			return ret;
		},
		
		dispatchGetModel : async function(bufferName, session, envId, dataBase, key, modelBufferDelegate)
		{
			var model = null;
			if(session!=null) model = await this.__getModelWithSession(bufferName, session, key, modelBufferDelegate);
			else if(dataBase!=null) model = await this.__getModelWithDataBase(bufferName, dataBase, key, modelBufferDelegate);
			else model = await this.__getModelWithEnvId(bufferName, envId, key, modelBufferDelegate);
			return model;
		},

		dispatchGetAllModels : async function(bufferName, session, envId, dataBase, modelBufferDelegate)
		{
			var modelList = null;
			if(session!=null) modelList = await this.__getAllModelsWithSession(bufferName, session, modelBufferDelegate);
			else if(dataBase!=null) modelList = await this.__getAllModelsWithDataBase(bufferName, dataBase, modelBufferDelegate);
			else modelList = await this.__getAllModelsWithEnvId(bufferName, envId, modelBufferDelegate);
			return modelList;
		}
	}
});