import "../../../core/database/IEnvDataBase";
import "../../../java/util/ArrayList";
import "../../../core/util/ObjectUtil";
import "../../../core/env/Environment";
import "../ClientBase";
import "./ClientTableInfoBuffer";
import "../util/RequestUtil";
import "../../../base/Base";
import "../../../core/database/bean/ParamBean";
import "../../../java/util/List";
var qx = require("xlsys-qx");

/**
 * ClientDataBase
 */
qx.Class.define("xlsys.client.base.database.ClientDataBase",
{
	extend : qx.core.Object,
	implement : [xlsys.core.database.IEnvDataBase],
	
	statics :
	{
		getInstance : function()
		{
			return new xlsys.client.base.database.ClientDataBase();
		}
	},
	
	construct: function()
	{
		this.autoCommit = true;
		this.transactionList = new xlsys.java.util.ArrayList();
	},
	
	members : 
	{
		autoCommit : null,
		transactionList : null,
		
		getDataBaseId : function()
		{
			throw new Error("Unsupport");
		},

		getEnvId : function()
		{
			return xlsys.core.util.ObjectUtil.objectToInt(xlsys.core.env.Environment.getInstance().getString(xlsys.client.base.ClientBase.ENV_ENV_ID));
		},
		
		getTableInfo : async function(tableName, useBuffer)
		{
			if(useBuffer) return await xlsys.client.base.database.ClientTableInfoBuffer.getInstance().getTableInfo(tableName);
			else return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_GET_TABLE_INFO, tableName);
		},
		
		getAllTableBaseInfo : async function(useBuffer)
		{
			if(useBuffer) return await xlsys.client.base.database.ClientTableInfoBuffer.getInstance().getAllTableBaseInfo();
			else return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_GET_ALL_TABLE_BASE_INFO, null);
		},
		
		getResultCount : async function(paramBean)
		{
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_GET_RESULT_COUNT, paramBean);
		},
		
		sqlSelectAsOneValue : async function(sqlOrPb)
		{
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_SQL_SELECT_AS_ONE_VALUE, sqlOrPb);
		},
		
		/**
		 * sqlSelect(selectSql);
		 * sqlSelect(paramBean);
		 * sqlSelect(paramBean, tryTableInfo);
		 * sqlSelect(paramBean, beginRowNum, endRowNum);
		 * sqlSelect(paramBean, sortColMap, beginRowNum, endRowNum);
		 */
		sqlSelect : async function()
		{
			var params = new Array();
			if(arguments.length==1)
			{
				// sqlSelect(selectSql);
				// sqlSelect(paramBean);
				params.push(arguments[0]);
			}
			else if(arguments.length==2)
			{
				// sqlSelect(paramBean, tryTableInfo);
				params.push(arguments[0]);
				params.push(arguments[1]);
			}
			else if(arguments.length==3)
			{
				// sqlSelect(paramBean, beginRowNum, endRowNum);
				params.push(arguments[0]);
				params.push(arguments[1]);
				params.push(arguments[2]);
			}
			else if(arguments.length==4)
			{
				// sqlSelect(paramBean, sortColMap, beginRowNum, endRowNum);
				params.push(arguments[0]);
				params.push(arguments[1]);
				params.push(arguments[2]);
				params.push(arguments[3]);
			}
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_SQL_SELECT, params);
		},
		
		/**
		 * sqlExecute(executeSql);
		 * sqlExecute(executeBean);
		 * sqlExecute(sqlBeanList);
		 * sqlExecute(paramBean);
		 */
		sqlExecute : async function(param)
		{
			if(this.autoCommit) return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_SQL_EXECUTE, param);
			else
			{
				if(qx.lang.Type.isString(param)) this.transactionList.add(new xlsys.core.database.bean.ParamBean(param));
				else if(qx.Interface.objectImplements(param, xlsys.java.util.List)) this.transactionList.addAll(param);
				else this.transactionList.add(param);
				return true;
			}
		},
		
		disableConstraint : async function(tableName, constraintName)
		{
			var params = new Array();
			params.push(tableName);
			params.push(constraintName);
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_DISABLE_CONSTRAINT, params);
		},
		
		enableConstraint : async function(tableName, constraintName)
		{
			var params = new Array();
			params.push(tableName);
			params.push(constraintName);
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_ENABLE_CONSTRAINT, params);
		},
		
		getLowerFunc : async function(srcExp)
		{
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_DBFUNC_LOWER, srcExp);
		},
		
		getUpperFunc : async function(srcExp)
		{
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_DBFUNC_UPPER, srcExp);
		},
		
		getByteLengthFunc : async function(srcExp)
		{
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_DBFUNC_BYTELENGTH, srcExp);
		},
		
		/**
		 * getSubStringFunc(srcExp, begin);
		 * getSubStringFunc(srcExp, begin, end);
		 */
		getSubStringFunc : async function(srcExp, begin)
		{
			var params = new Array();
			params.push(srcExp);
			params.push(begin);
			if(arguments.length==3) params.push(arguments[2]);
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_DBFUNC_SUBSTRING, params);
		},

		getToDateFunc : async function(date)
		{
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_DBFUNC_TODATE, date);
		},
		
		getDateFormatFunc : async function(srcExp, format)
		{
			var params = new Array();
			params.push(srcExp);
			params.push(format);
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_DBFUNC_DATEFORMAT, params);
		},
		
		getSplitedParamBean : async function(paramBean, beginRowNum, endRowNum)
		{
			var params = new Array();
			params.push(paramBean);
			params.push(beginRowNum);
			params.push(endRowNum);
			return await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_GET_SPLITED_PARAM_BEAN, params);
		},
		
		commit : async function()
		{
			if(!this.autoCommit&&this.transactionList.size()>0)
			{
				var ret = await xlsys.client.base.util.RequestUtil.simpleRequest(xlsys.base.Base.COMMAND_SQL_EXECUTE, this.transactionList);
				if(ret==true)
				{
					this.transactionList.clear();
					return ret;
				}
				else throw ret;
			}
			else return true;
		},

		rollback : async function()
		{
			if(!this.autoCommit&&this.transactionList.size()>0) this.transactionList.clear();
			return;
		},

		setAutoCommit : async function(autoCommit)
		{
			if(autoCommit&&!this.autoCommit) await this.commit();
			this.autoCommit = autoCommit;
			return;
		},

		getAutoCommit : function()
		{
			return this.autoCommit;
		},

		close : async function()
		{
			if(!this.autoCommit) await this.commit(callback);
			return;
		},

		isClose : function()
		{
			return false;
		}
	}
});