import "../buffer/UnstorageBuffer";
import "../buffer/IClientBaseBuffer";
import "../../../java/util/HashMap";
import "../../../core/util/ObjectUtil";
import "../../../core/env/Environment";
import "../ClientBase";
import "../../../core/buffer/BufferManager";
import "../../../core/Core";
import "./ClientDataBase";
import "../../../core/database/IDataBase";
import "../../../core/log/Log";
var qx = require("xlsys-qx");

/**
 * ClientDataBase的表信息缓存对象
 */
qx.Class.define("xlsys.client.base.database.ClientTableInfoBuffer",
{
	type : "singleton",
	extend : xlsys.client.base.buffer.UnstorageBuffer,
	implement : [xlsys.client.base.buffer.IClientBaseBuffer],
	
	construct : function()
	{
		this.base(arguments);
		this._tableInfoBuffer = new xlsys.java.util.HashMap();
		this._allTableBaseInfo = new xlsys.java.util.HashMap();
		var envId = xlsys.core.util.ObjectUtil.objectToInt(xlsys.core.env.Environment.getInstance().getString(xlsys.client.base.ClientBase.ENV_ENV_ID));
		xlsys.core.buffer.BufferManager.getInstance().registerBuffer("_ENV"+xlsys.core.Core.BUFFER_TABLE_INFO_PREFIX+envId, this);
	},
	
	members : 
	{
		_tableInfoBuffer : null,
		_allTableBaseInfo : null,
		
		getAllTableBaseInfo : async function()
		{
			if(this._allTableBaseInfo.isEmpty())
			{
				var dataBase = xlsys.client.base.database.ClientDataBase.getInstance();
				this._allTableBaseInfo.putAll(await dataBase.getAllTableBaseInfo(false));
			}
			return this._allTableBaseInfo;
		},
		
		getTableInfo : async function(tableName)
		{
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(xlsys.core.database.IDataBase.BUFFER_KEY_TABLE_NAME, tableName);
			var envId = xlsys.core.util.ObjectUtil.objectToInt(xlsys.core.env.Environment.getInstance().getString(xlsys.client.base.ClientBase.ENV_ENV_ID));
			return await this.getBufferData(envId, "_ENV"+xlsys.core.Core.BUFFER_TABLE_INFO_PREFIX+envId, paramMap);
		},

		reloadDataDirectly : async function(envId, bufferName, paramMap, forceLoad)
		{
			var tableName = null;
			if(paramMap!=null) tableName = paramMap.get(xlsys.core.database.IDataBase.BUFFER_KEY_TABLE_NAME);
			if(tableName!=null) tableName = tableName.toLowerCase();
			if(tableName!=null) this._tableInfoBuffer.remove(tableName);
			else
			{
				this._tableInfoBuffer.clear();
				this._allTableBaseInfo.clear();
			}
			if(forceLoad) await this.doGetBufferData(envId, bufferName, paramMap);
			return;
		},

		getBufferData : async function(envId, bufferName, paramMap)
		{
			var tableName = null;
			if(paramMap!=null) tableName = paramMap.get(xlsys.core.database.IDataBase.BUFFER_KEY_TABLE_NAME);
			if(tableName!=null) tableName = tableName.toLowerCase();
			var ret = null;
			if(tableName!=null) ret = this._tableInfoBuffer.get(tableName);
			else ret = this._tableInfoBuffer;
			if(ret==null)
			{
				ret = await this.doGetBufferData(envId, bufferName, paramMap);
				if(tableName!=null) this._tableInfoBuffer.put(tableName, ret);
				else this._tableInfoBuffer.putAll(ret);
			}
			return ret;
		},
		
		reloadAllData : async function(envId, bufferName, forceLoad)
		{
			this._tableInfoBuffer.clear();
			this._allTableBaseInfo.clear();
			if(forceLoad) await this.reloadDataDirectly(envId, bufferName, null, forceLoad);
			return;
		},

		reloadData : async function(envId, bufferName, paramMap, forceLoad)
		{
			return await this.reloadDataDirectly(envId, bufferName, paramMap, forceLoad);
		},
		
		doGetBufferData : async function(envId, bufferName, paramMap)
		{
			var ret = null;
			var tableName = null;
			if(paramMap!=null) tableName = paramMap.get(xlsys.core.database.IDataBase.BUFFER_KEY_TABLE_NAME);
			if(tableName!=null)
			{
				xlsys.core.log.Log.getInstance().printlnDebug("GetTableInfo [bufferName:" + ("_ENV"+xlsys.core.Core.BUFFER_TABLE_INFO_PREFIX+envId) + ", key:"+tableName+"]");
				return await xlsys.client.base.database.ClientDataBase.getInstance().getTableInfo(tableName, false);
			}
			else
			{
				xlsys.core.log.Log.getInstance().printlnDebug("GetAllTableInfo [bufferName:" + ("_ENV"+xlsys.core.Core.BUFFER_TABLE_INFO_PREFIX+envId) + "]");
				var allTableInfo = await this.getAllTableBaseInfo();
				var keys = allTableInfo.keys();
				var allPromises = new Array();
				for(var i in keys)
				{
					var tempTable = keys[i];
					allPromises.push(xlsys.client.base.database.ClientDataBase.getInstance().getTableInfo(tempTable, false));
				}
				let tableInfos = await Promise.all(allPromises);
				for(var i in tableInfos)
				{
					this._tableInfoBuffer.put(tableInfos[i].getTableName(), tableInfos[i]);
				}
				return this._tableInfoBuffer;
			}
		}
	}
});