import "../../../base/buffer/ModelBuffer";
import "../../../../business/core/BusinessCore";
import "../../../../core/buffer/BufferManager";
import "./CBCModelBufferDelegate";
import "../../../base/session/SessionManager";
import "../../../../java/util/Map";
import "../../../../java/util/LinkedHashMap";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.client.business.core.buffer.CBCModelBuffer",
{
	type : "singleton",
	extend : xlsys.client.base.buffer.ModelBuffer,
	
	statics : 
	{
		_getModelListByBufferName : async function(bufferName)
		{
			var session = xlsys.client.base.session.SessionManager.getInstance().getCurrentSession();
			return await this.self(arguments).getInstance().getAllModels(bufferName, session);
		},
		
		/**
		 * 获取模型的码名映射Map. 支持以下方法:
		 * 
		 * <li> getModelCodeNameMap(paramMap); 其中paramMap需包含以下方法中的参数
		 * <li> getModelCodeNameMap(bufferName);
		 * <li> getModelCodeNameMap(modelList);
		 * <li> getModelCodeNameMap(bufferName, codeFieldNameOrIndex, nameFieldNameOrIndex);
		 * <li> getModelCodeNameMap(modelList, codeFieldNameOrIndex, nameFieldNameOrIndex);
		 */
		getModelCodeNameMap : async function()
		{
			var modelList = null;
			var codeFieldNameOrIndex = null;
			var nameFieldNameOrIndex = null;
			if(arguments.length==1)
			{
				if(qx.lang.Type.isString(arguments[0]))
				{
					modelList = await this._getModelListByBufferName(arguments[0]);
				}
				else if(qx.Class.implementsInterface(arguments[0], xlsys.java.util.Map))
				{
					var paramMap = arguments[0];
					modelList = paramMap.get("modelList");
					if(modelList==null) modelList = await this._getModelListByBufferName(paramMap.get("bufferName"));
					codeFieldNameOrIndex = paramMap.get("codeFieldNameOrIndex");
					nameFieldNameOrIndex = paramMap.get("nameFieldNameOrIndex");
				}
				else modelList = arguments[0];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isString(arguments[0])) modelList = await this._getModelListByBufferName(arguments[0]);
				else modelList = arguments[0];
				codeFieldNameOrIndex = arguments[1];
				nameFieldNameOrIndex = arguments[2];
			}
			var codeFieldName = null;
			if(codeFieldNameOrIndex==null) codeFieldNameOrIndex = 0;
			if(qx.lang.Type.isString(codeFieldNameOrIndex)) codeFieldName = codeFieldNameOrIndex;
			else
			{
				var fieldArr = modelList.get(0).getSerializableMembers();
				codeFieldName = fieldArr[codeFieldNameOrIndex];
			}
			var nameFieldName = null;
			if(nameFieldNameOrIndex==null) nameFieldNameOrIndex = 1;
			if(qx.lang.Type.isString(nameFieldNameOrIndex)) nameFieldName = nameFieldNameOrIndex;
			else
			{
				var fieldArr = modelList.get(0).getSerializableMembers();
				nameFieldName = fieldArr[nameFieldNameOrIndex];
			}
			var codeNameMap = new xlsys.java.util.LinkedHashMap();
			for(var i=0;i<modelList.size();++i)
			{
				var model = modelList.get(i);
				codeNameMap.put(model[codeFieldName], model[nameFieldName]);
			}
			return codeNameMap;
		}
	},
	
	construct : function()
	{
		this.base(arguments);
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_IDENTITY_MODEL, "id");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_USER_MODEL, "userid");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_DEPARTMENT_MODEL, "deptid");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_POSITION_MODEL, "pstid");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_RIGHT_MODEL, "righttype");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_FLOW_MODEL, "flowid");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_AUTHORIZE_MODEL, "arid");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_CODEALLOC_MODEL, "caid");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_FLOWCODEALLOC_MODEL, "fcaid");
		this._registerBuffer(xlsys.business.core.BusinessCore.BUFFER_DICT_MODEL, "dictid");
	},
	
	members : 
	{
		_registerBuffer : function(bufferName, keyColumn)
		{
			xlsys.core.buffer.BufferManager.getInstance().registerBuffer(bufferName, this);
			this.bindModelBufferDelegate(bufferName, new xlsys.client.business.core.buffer.CBCModelBufferDelegate(bufferName, keyColumn));
		}
	}
});
	