import "../../java/util/HashMap";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.buffer.BufferManager",
{
	type : "singleton",
	extend : qx.core.Object,

	construct : function()
	{
		this.base(arguments);
		this.bufferMap = new xlsys.java.util.HashMap();
	},
	
	members : 
	{
		bufferMap : null,
		
		registerBuffer : function(bufferName, buffer)
		{
			if(bufferName!=null&&buffer!=null) this.bufferMap.put(bufferName, buffer);
		},
		
		cancelRegisterBuffer : function(bufferName)
		{
			if(bufferName!=null) this.bufferMap.remove(bufferName);
		},
		
		isRegistered : function(bufferName)
		{
			return this.bufferMap.containsKey(bufferName);
		},
		
		/**
		 * reloadBuffer(envId, bufferName);
		 * reloadBuffer(envId, bufferName, paramMap);
		 * reloadBuffer(envId, bufferName, paramMap, forceLoad);
		 */
		reloadBuffer : async function(envId, bufferName, paramMap, forceLoad)
		{
			if(forceLoad==null) forceLoad = false;
			if(bufferName==null) return;
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null) return;
			if(paramMap!=null) return await buffer.reloadData(envId, bufferName, paramMap, forceLoad);
			else return await buffer.reloadAllData(envId, bufferName, forceLoad);
		},
		
		/**
		 * reloadAllBuffer(envId);
		 * reloadAllBuffer(envId, forceLoad);
		 */
		reloadAllBuffer : async function(envId, forceLoad)
		{
			if(forceLoad==null) forceLoad = false;
			var keys = this.bufferMap.keys();
			var allPromises = new Array();
			for(var i=0;i<keys.length;++i)
			{
				var bufferName = keys[i];
				allPromises.push(this.reloadBuffer(envId, bufferName, null, forceLoad));
			}
			await Promise.all(allPromises);
			return;
		},
		
		getAllRegisteredBufferNames : function()
		{
			return this.bufferMap.keys();
		},
		
		updateBufferVersion : async function(envId, bufferName)
		{
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null) return -1;
			return await buffer.updateCurrentVersion(envId, bufferName);
		},
		
		getBufferVersion : async function(envId, bufferName)
		{
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null) return -1;
			return await buffer.getCurrentVersion(envId, bufferName);
		},
		
		getLocalBufferVersion : async function(envId, bufferName)
		{
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null) return -1;
			return await buffer.getLocalStorageVersion(envId, bufferName);
		}
	}
});
