import "./DataSet";
import "./IStorableDataSet";
import "../../java/util/HashSet";
import "../../java/util/ArrayList";
import "./StorableDataSetRow";
import "../util/ObjectUtil";
import "../../java/util/HashMap";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.dataset.AbstractStorableDataSet",
{
	type : "abstract",
	extend : xlsys.core.dataset.DataSet,
	implement : [xlsys.core.dataset.IStorableDataSet],
	
	events : 
	{
		// async supported
		"beforeOpen" : "qx.event.type.Event",
		// async supported
		"afterOpen" : "qx.event.type.Event",
		// async supported
		"openFinally" : "qx.event.type.Event",
		// async supported
		"beforeSave" : "qx.event.type.Event",
		// async supported
		"afterSave" : "qx.event.type.Event",
		// async supported
		"saveFinally" : "qx.event.type.Event"
	},
	
	construct : function()
	{
		this.base(arguments);
		this.rowClassName = "xlsys.core.dataset.StorableDataSetRow";
		this.colClassName = "xlsys.core.dataset.StorableDataSetColumn";
		this.cellClassName = "xlsys.core.dataset.DataSetCell";
		this.changed = false;
		this.saveTransaction = true;
		this.lockChange = 0;
	},
	
	members : 
	{
		changed : null,
		noneedSaveCols : null,
		toBeDeleteRows : null,
		lockChange : null,
		saveTransaction : null,
		
		getSerializableMembers : function()
		{
			var arr = this.base(arguments);
			arr.push("changed");
			arr.push("noneedSaveCols");
			arr.push("toBeDeleteRows");
			return arr;
		},
		
		refresh : async function()
		{
			if(this.toBeDeleteRows!=null) this.toBeDeleteRows.clear();
			this.changed = false;
			this.getRows().clear();
			this.getCols().clear();
			this.rowCursor = -1;
			this.colCursor = -1;
			await this.open();
		},
		
		isChanged : function()
		{
			return this.changed;
		},
		
		setLockChange : function(lockChange)
		{
			if(lockChange) ++this.lockChange;
			else --this.lockChange;
			if(this.lockChange<0) this.lockChange = 0;
		},

		isLockChange : function()
		{
			return this.lockChange>0;
		},

		setChanged : function(changed)
		{
			if(!this.isLockChange()) this.changed = changed;
		},

		setSaveTransaction : function(saveTransaction)
		{
			this.saveTransaction = saveTransaction;
		},

		isSaveTransaction : function()
		{
			return this.saveTransaction;
		},

		addNoneedSaveCol : function(colName)
		{
			if(this.noneedSaveCols==null) this.noneedSaveCols = new xlsys.java.util.HashSet();
			this.noneedSaveCols.add(colName);
		},

		getNoneedSaveCols : function()
		{
			return this.noneedSaveCols;
		},

		hasToBeDeleteRow : function()
		{
			return this.toBeDeleteRows!=null&&!this.toBeDeleteRows.isEmpty();
		},

		getToBeDeleteRows : function()
		{
			return this.toBeDeleteRows;
		},

		removeRow : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			if(!this.isLockChange())
			{
				if(this.toBeDeleteRows==null) this.toBeDeleteRows = new xlsys.java.util.ArrayList();
				this.toBeDeleteRows.add(this.getRow(rowAt));
			}
			var row = this.rows.get(rowAt);
			if(!this.isEventLock("beforeRemoveRow")) this.fireDataEvent("beforeRemoveRow", row);
			this.rows.removeAt(rowAt);
			this.setChanged(true);
			if(!this.isEventLock("afterRemoveRow")) this.fireDataEvent("afterRemoveRow", row);
			return row;
		},

		setValue : function()
		{
			var rowAt = null;
			var colAt = null;
			var value = null;
			if(arguments.length==1)
			{
				rowAt = this.rowCursor;
				colAt = this.colCursor;
				value = arguments[0];
			}
			else if(arguments.length==2)
			{
				rowAt = this.rowCursor;
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
				value = arguments[1];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0])) rowAt = arguments[0];
				else rowAt = this.getRowAt(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colAt = arguments[1];
				else colAt = this.getColAt(arguments[1]);
				value = arguments[2];
			}
			this.checkRowRegion(rowAt);
			this.checkColRegion(colAt);
			var oldValue = this.rows.get(rowAt).getCell(colAt).getContent();
			if(oldValue==value||(oldValue!=null&&value!=null&&xlsys.core.util.ObjectUtil.hashcode(oldValue)==xlsys.core.util.ObjectUtil.hashcode(value))) return false;
			var dataMap = new xlsys.java.util.HashMap();
			dataMap.put("rowAt", rowAt);
			dataMap.put("colAt", colAt);
			dataMap.put("oldValue", oldValue);
			dataMap.put("value", value);
			if(!this.isEventLock("beforePostValue")) this.fireDataEvent("beforePostValue", dataMap);
			this.rows.get(rowAt).getCell(colAt).setContent(value);
			if(!this.isLockChange()) this.setRowStatusAfterSetValueSuccess(rowAt, colAt);
			this.setChanged(true);
			if(!this.isEventLock("afterPostValue")) this.fireDataEvent("afterPostValue", dataMap);
			return true;
		},
		
		setRowStatusAfterSetValueSuccess : function(rowAt, colAt)
		{
			throw new Error("Abstract method call");
		},

		createCol : function()
		{
			var index = null;
			if(arguments.length==0) index = this.cols.size();
			else index = arguments[0];
			var temp = this.base(arguments, index);
			this.setChanged(true);
			return temp;
		},

		createRow : function()
		{
			var index = null;
			if(arguments.length==0) index = this.rows.size();
			else index = arguments[0];
			var row = xlsys.core.util.ReflectUtil.newInstance(this.rowClassName);
			if(!this.isLockChange()) row.setStatus(xlsys.core.dataset.StorableDataSetRow.STATUS_FOR_NEW);
			row.setCellClassName(this.cellClassName);
			var colCount = this.getColCount();
			for(var i=0;i<colCount;++i)
			{
				var cell = xlsys.core.util.ReflectUtil.newInstance(this.cellClassName);
				row.addCell(cell);
			}
			this.addRow(index, row);
			this.setChanged(true);
			return row;
		},

		addCol : function()
		{
			var index = null;
			var col = null;
			if(arguments.length==1)
			{
				index = this.cols.size();
				col = arguments[0];
			}
			else if(arguments.length==2)
			{
				index = arguments[0];
				col = arguments[1];
			}
			var temp = this.base(arguments, index, col);
			this.setChanged(true);
			return temp;
		},

		addRow : function()
		{
			var index = null;
			var row = null;
			if(arguments.length==1)
			{
				index = this.rows.size();
				row = arguments[0];
			}
			else if(arguments.length==2)
			{
				index = arguments[0];
				row = arguments[1];
			}
			var temp = this.base(arguments, index, row);
			if(!this.isLockChange()) temp.setStatus(xlsys.core.dataset.StorableDataSetRow.STATUS_FOR_NEW);
			this.setChanged(true);
			return temp;
		}
	}
});