import "../../java/io/Serializable";
import "./IDataSet";
import "../../java/util/ArrayList";
import "../../java/util/HashMap";
import "../util/Pair";
import "../util/ObjectUtil";
import "../util/ReflectUtil";
import "../event/MAsyncListener";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.dataset.DataSet",
{
	extend : xlsys.java.io.Serializable,
	implement : [xlsys.core.dataset.IDataSet],
	include : [xlsys.core.event.MAsyncListener],
	
	events : 
	{
		"beforeCursorChange" : "qx.event.type.Data",
		"afterCursorChange" : "qx.event.type.Data",
		// async supported in IVirtualizableDataSet
		"beforePostValue" : "qx.event.type.Data",
		// async supported in IVirtualizableDataSet
		"afterPostValue" : "qx.event.type.Data",
		// async supported in IVirtualizableDataSet
		"beforeRemoveRow" : "qx.event.type.Data",
		// async supported in IVirtualizableDataSet
		"afterRemoveRow" : "qx.event.type.Data",
		"beforeRemoveCol" : "qx.event.type.Data",
		"afterRemoveCol" : "qx.event.type.Data",
		"beforeAddRow" : "qx.event.type.Data",
		"afterAddRow" : "qx.event.type.Data"
	},
	
	construct : function()
	{
		this.rowCursor = -1;
		this.colCursor = -1;
		this.rows = new xlsys.java.util.ArrayList();
		this.cols = new xlsys.java.util.ArrayList();
		this.colMap = new xlsys.java.util.HashMap();
		this.rowClassName = "xlsys.core.dataset.DataSetRow";
		this.colClassName = "xlsys.core.dataset.DataSetColumn";
		this.cellClassName = "xlsys.core.dataset.DataSetCell";
		this.eventLockMap = new xlsys.java.util.HashMap();
	},
	
	members : 
	{
		rowCursor : null,
		colCursor : null,
		rows : null,
		cols : null,
		colMap : null,
		rowClassName : null,
		colClassName : null,
		cellClassName : null,
		eventLockMap : null,
		
		getSerializableMembers : function()
		{
			var arr = new Array();
			arr.push("rows");
			arr.push("cols");
			return arr;
		},
		
		refresh : async function() {},
		
		getCursor : function()
		{
			return new xlsys.core.util.Pair(this.rowCursor, this.colCursor);
		},
		
		getRowCursor : function()
		{
			return this.rowCursor;
		},
		
		getColCursor : function()
		{
			return this.colCursor;
		},
		
		/**
		 * checkRowRegion(rowAt);
		 * checkRowRegion(rowAt, throwable);
		 */
		checkRowRegion : function(rowAt, throwable)
		{
			if(throwable==null) throwable = true;
			if(rowAt<0||rowAt>=this.rows.size())
			{
				if(throwable) throw new Error("rowAt : " + rowAt);
				return false;
			}
			return true;
		},
		
		/**
		 * checkColRegion(colAt);
		 * checkColRegion(colAt, throwable);
		 */
		checkColRegion : function(colAt, throwable)
		{
			if(throwable==null) throwable = true;
			if(colAt<0||colAt>=this.cols.size())
			{
				if(throwable) throw new Error("colAt : " + colAt);
				return false;
			}
			return true;
		},

		/**
		 * gotoRow(row);
		 * gotoRow(rowAt);
		 */
		gotoRow : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			if(this.rowCursor==rowAt) return true;
			return this.gotoCell(rowAt, this.colCursor);
		},
		
		/**
		 * gotoCol(col);
		 * gotoCol(colAt);
		 */
		gotoCol : function(colOrColAt)
		{
			var colAt = null;
			if(qx.lang.Type.isNumber(colOrColAt)) colAt = colOrColAt;
			else colAt = this.getColAt(colOrColAt);
			if(this.colCursor==colAt) return true;
			return this.gotoCell(this.rowCursor, colAt);
		},
		
		/**
		 * gotoCell(row, colAt);
		 * gotoCell(row, col);
		 * gotoCell(rowAt, col);
		 * gotoCell(rowAt, colAt);
		 */
		gotoCell : function(rowOrRowAt, colOrColAt)
		{
			var rowAt = null;
			var colAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			if(qx.lang.Type.isNumber(colOrColAt)) colAt = colOrColAt;
			else colAt = this.getColAt(colOrColAt);
			if(rowAt!=-1) this.checkRowRegion(rowAt);
			if(colAt!=-1) this.checkColRegion(colAt);
			var oldRowAt = this.rowCursor;
			var oldColAt = this.colCursor;
			var dataMap = new xlsys.java.util.HashMap();
			dataMap.put("rowAt", rowAt);
			dataMap.put("oldRowAt", oldRowAt);
			dataMap.put("colAt", colAt);
			dataMap.put("oldColAt", oldColAt);
			if(!this.isEventLock("beforeCursorChange")) this.fireDataEvent("beforeCursorChange", dataMap);
			this.rowCursor = rowAt;
			this.colCursor = colAt;
			if(!this.isEventLock("afterCursorChange")) this.fireDataEvent("afterCursorChange", dataMap);
			return true;
		},

		/**
		 * getRow();
		 * getRow(rowAt);
		 */
		getRow : function()
		{
			var rowAt = null;
			if(arguments.length>0) rowAt = arguments[0];
			else rowAt = this.rowCursor;
			if(rowAt==-1) return null;
			if(!this.checkRowRegion(rowAt, false)) return null;
			return this.rows.get(rowAt);
		},
		
		/**
		 * getColumn();
		 * getColumn(colAt);
		 * getColumn(colName);
		 */
		getColumn : function()
		{
			var colAt = null;
			if(arguments.length>0)
			{
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
			}
			else colAt = this.colCursor;
			if(colAt==-1) return null;
			if(!this.checkColRegion(colAt, false)) return null;
			return this.cols.get(colAt);
		},
		
		/**
		 * getValue();
		 * getValue(colAt);
		 * getValue(colName);
		 * getValue(row, colAt);
		 * getValue(row, colName);
		 * getValue(rowAt, colName);
		 * getValue(rowAt, colAt);
		 */
		getValue : function()
		{
			var rowAt = null;
			var colAt = null;
			if(arguments.length==0)
			{
				rowAt = this.rowCursor;
				colAt = this.colCursor;
			}
			else if(arguments.length==1)
			{
				rowAt = this.rowCursor;
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
			}
			else
			{
				if(qx.lang.Type.isNumber(arguments[0])) rowAt = arguments[0];
				else rowAt = this.getRowAt(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colAt = arguments[1];
				else colAt = this.getColAt(arguments[1]);
			}
			this.checkRowRegion(rowAt);
			this.checkColRegion(colAt);
			var row = this.rows.get(rowAt);
			return row.getCell(colAt).getContent();
		},
		
		/**
		 * asyncGetValue();
		 * asyncGetValue(colAt);
		 * asyncGetValue(colName);
		 * asyncGetValue(row, colAt);
		 * asyncGetValue(row, colName);
		 * asyncGetValue(rowAt, colName);
		 * asyncGetValue(rowAt, colAt);
		 */
		asyncGetValue : async function()
		{
			var rowAt = null;
			var colAt = null;
			if(arguments.length==0)
			{
				rowAt = this.rowCursor;
				colAt = this.colCursor;
			}
			else if(arguments.length==1)
			{
				rowAt = this.rowCursor;
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
			}
			else
			{
				if(qx.lang.Type.isNumber(arguments[0])) rowAt = arguments[0];
				else rowAt = this.getRowAt(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colAt = arguments[1];
				else colAt = this.getColAt(arguments[1]);
			}
			return this.getValue(rowAt, colAt);
		},
		
		/**
		 * setValue(value);
		 * setValue(colName, value);
		 * setValue(colAt, value);
		 * setValue(row, colName, value);
		 * setValue(row, colAt, value);
		 * setValue(rowAt, colName, value);
		 * setValue(rowAt, colAt, value);
		 */
		setValue : function()
		{
			var rowAt = null;
			var colAt = null;
			var value = null;
			if(arguments.length==1)
			{
				rowAt = this.rowCursor;
				colAt = this.colCursor;
				value = arguments[0];
			}
			else if(arguments.length==2)
			{
				rowAt = this.rowCursor;
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
				value = arguments[1];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0])) rowAt = arguments[0];
				else rowAt = this.getRowAt(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colAt = arguments[1];
				else colAt = this.getColAt(arguments[1]);
				value = arguments[2];
			}
			this.checkRowRegion(rowAt);
			this.checkColRegion(colAt);
			var oldValue = this.rows.get(rowAt).getCell(colAt).getContent();
			if(oldValue==value||(oldValue!=null&&value!=null&&xlsys.core.util.ObjectUtil.hashcode(oldValue)==xlsys.core.util.ObjectUtil.hashcode(value))) return false;
			var dataMap = new xlsys.java.util.HashMap();
			dataMap.put("rowAt", rowAt);
			dataMap.put("colAt", colAt);
			dataMap.put("oldValue", oldValue);
			dataMap.put("value", value);
			if(!this.isEventLock("beforePostValue")) this.fireDataEvent("beforePostValue", dataMap);
			this.rows.get(rowAt).getCell(colAt).setContent(value);
			if(!this.isEventLock("afterPostValue")) this.fireDataEvent("afterPostValue", dataMap);
			return true;
		},
		
		/**
		 * asyncSetValue(value);
		 * asyncSetValue(colName, value);
		 * asyncSetValue(colAt, value);
		 * asyncSetValue(row, colName, value);
		 * asyncSetValue(row, colAt, value);
		 * asyncSetValue(rowAt, colName, value);
		 * asyncSetValue(rowAt, colAt, value);
		 */
		asyncSetValue : async function()
		{
			var rowAt = null;
			var colAt = null;
			var value = null;
			if(arguments.length==1)
			{
				rowAt = this.rowCursor;
				colAt = this.colCursor;
				value = arguments[0];
			}
			else if(arguments.length==2)
			{
				rowAt = this.rowCursor;
				if(qx.lang.Type.isNumber(arguments[0])) colAt = arguments[0];
				else colAt = this.getColAt(arguments[0]);
				value = arguments[1];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0])) rowAt = arguments[0];
				else rowAt = this.getRowAt(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colAt = arguments[1];
				else colAt = this.getColAt(arguments[1]);
				value = arguments[2];
			}
			return this.setValue(rowAt, colAt, value);
		},
		
		getRows : function()
		{
			var tempRows = new xlsys.java.util.ArrayList();
			tempRows.addAll(this.rows);
			return tempRows;
		},
		
		_getFactRows : function()
		{
			return this.rows;
		},
		
		getCols : function()
		{
			var tempCols = new xlsys.java.util.ArrayList();
			tempCols.addAll(this.cols);
			return tempCols;
		},
		
		_getFactCols : function()
		{
			return this.cols;
		},
		
		getColName : function(colAt)
		{
			this.checkColRegion(colAt);
			return this.cols.get(colAt).getColumnName();
		},

		setRowClassName : function(rowClassName)
		{
			this.rowClassName = rowClassName;
		},

		setColClassName : function(colClassName)
		{
			this.colClassName = colClassName;
		},

		setCellClassName : function(cellClassName)
		{
			this.cellClassName = cellClassName;
		},

		getRowClassName : function()
		{
			return this.rowClassName;
		},

		getColClassName : function()
		{
			return this.colClassName;
		},

		getCellClassName : function()
		{
			return this.cellClassName;
		},

		/**
		 * setEventLock(lock);
		 * setEventLock(eventName, lock);
		 */
		setEventLock : function()
		{
			var eventName = null;
			var lock = null;
			if(arguments.length==1) lock = arguments[0];
			else if(arguments.length==2)
			{
				eventName = arguments[0];
				lock = arguments[1];
			}
			var eventNames = null;
			if(eventName==null) eventNames = xlsys.core.util.ReflectUtil.getValidEventName(xlsys.core.util.ObjectUtil.getClass(this)).toArray();
			else
			{
				eventNames = new Array();
				eventNames.add(eventName);
			}
			for(var i in eventNames)
			{
				var name = eventNames[i];
				var count = this.eventLockMap.get(name);
				if(count==null) count = 0;
				if(lock) this.eventLockMap.put(name, ++count);
				else
				{
					this.eventLockMap.put(name, --count);
					if(count<=0) this.eventLockMap.remove(name);
				}
			}
		},

		isEventLock : function(eventName)
		{
			var isLock = false;
			var count = this.eventLockMap.get(eventName);
			if(count!=null) isLock = (count!=0);
			return isLock;
		},
		
		getRowAt : function(row)
		{
			if(row==null) return -1;
			return this.rows.indexOf(row);
		},

		/**
		 * getColAt(col);
		 * getColAt(colName);
		 */
		getColAt : function(colOrColName)
		{
			if(this.colMap.isEmpty()) this._buildColMap();
			if(colOrColName==null) return -1;
			else if(qx.lang.Type.isString(colOrColName))
			{
				var colAt = this.colMap.get(colOrColName);
				return colAt==null?-1:colAt;
			}
			else return this.cols.indexOf(colOrColName);
		},
		
		_buildColMap : function()
		{
			let tempColMap = new xlsys.java.util.HashMap();
			for(let i=0;i<this.cols.size();++i)
			{
				let colName = this.cols.get(i).getColumnName();
				if(colName==null) return;
				tempColMap.put(colName, i);
			}
			this.colMap.clear();
			this.colMap.putAll(tempColMap);
		},
		
		/**
		 * createCol();
		 * createCol(index);
		 */
		createCol : function()
		{
			var index = null;
			if(arguments.length==0) index = this.cols.size();
			else index = arguments[0];
			var col = xlsys.core.util.ReflectUtil.newInstance(this.colClassName);
			this.addCol(index, col);
			return col;
		},

		/**
		 * createRow();
		 * createRow(index);
		 */
		createRow : function()
		{
			var index = null;
			if(arguments.length==0) index = this.rows.size();
			else index = arguments[0];
			var row = xlsys.core.util.ReflectUtil.newInstance(this.rowClassName);
			row.setCellClassName(this.cellClassName);
			var colCount = this.getColCount();
			for(var i=0;i<colCount;++i)
			{
				var cell = xlsys.core.util.ReflectUtil.newInstance(this.cellClassName);
				row.addCell(cell);
			}
			this.addRow(index, row);
			return row;
		},
		
		/**
		 * addCol(col);
		 * addCol(index, col);
		 */
		addCol : function()
		{
			var index = null;
			var col = null;
			if(arguments.length==1)
			{
				index = this.cols.size();
				col = arguments[0];
			}
			else if(arguments.length==2)
			{
				index = arguments[0];
				col = arguments[1];
			}
			this.cols.add(index, col);
			if(!this.rows.isEmpty())
			{
				for(var i=0;i<this.rows.size();++i)
				{
					var cell = xlsys.core.util.ReflectUtil.newInstance(this.cellClassName);
					this.rows.get(i).getCells().add(index, cell);
				}
			}
			this.gotoCol(col);
			return col;
		},

		/**
		 * addRow(row);
		 * addRow(index, row);
		 */
		addRow : function()
		{
			var index = null;
			var row = null;
			if(arguments.length==1)
			{
				index = this.rows.size();
				row = arguments[0];
			}
			else if(arguments.length==2)
			{
				index = arguments[0];
				row = arguments[1];
			}
			if(!this.isEventLock("beforeAddRow")) this.fireDataEvent("beforeAddRow", row);
			if(index<=this.rowCursor) ++this.rowCursor;
			this.rows.add(index, row);
			if(!this.isEventLock("afterAddRow")) this.fireDataEvent("afterAddRow", row);
			this.gotoRow(row);
			return row;
		},

		getRowCount : function()
		{
			return this.rows.size();
		},
		
		getColCount : function()
		{
			return this.cols.size();
		},
		
		/**
		 * removeRow(row);
		 * removeRow(rowAt);
		 */
		removeRow : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			var row = this.rows.get(rowAt);
			if(!this.isEventLock("beforeRemoveRow")) this.fireDataEvent("beforeRemoveRow", row);
			this.rows.removeAt(rowAt);
			if(!this.isEventLock("afterRemoveRow")) this.fireDataEvent("afterRemoveRow", row);
			return row;
		},
		
		/**
		 * asyncRemoveRow(row);
		 * asyncRemoveRow(rowAt);
		 */
		asyncRemoveRow : async function(rowOrRowAt)
		{
			return this.removeRow(rowOrRowAt);
		},
		
		clearRows : function()
		{
			var tempRows = this.rows.toArray();
			for(var i in tempRows)
			{
				this.removeRow(tempRows[i]);
			}
			this.rowCursor = -1;
		},
		
		/**
		 * removeCol(col);
		 * removeCol(colAt);
		 */
		removeCol : function(colOrColAt)
		{
			var colAt = null;
			if(qx.lang.Type.isNumber(colOrColAt)) colAt = colOrColAt;
			else colAt = this.getColAt(colOrColAt);
			this.checkColRegion(colAt);
			var col = this.cols.get(colAt);
			if(!this.isEventLock("beforeRemoveCol")) this.fireDataEvent("beforeRemoveCol", col);
			this.cols.removeAt(colAt);
			for(var i=0;i<this.rows.size();++i)
			{
				this.rows.get(i).getCells().removeAt(colAt);
			}
			this._buildColMap();
			if(!this.isEventLock("afterRemoveCol")) this.fireDataEvent("afterRemoveCol", col);
			return col;
		},
		
		cloneOf : function(anotherDataSet)
		{
			this.rows.clear();
			this.rows.addAll(anotherDataSet._getFactRows());
			this.cols.addAll(anotherDataSet._getFactCols());
			this.colMap.clear();
		}
	}
});