import "../../java/io/Serializable";
import "./IDataSetRow";
import "../../java/util/ArrayList";
import "../../java/util/HashMap";
import "../util/ReflectUtil";
var qx = require("xlsys-qx");

/**
 * 数据集行
 */
qx.Class.define("xlsys.core.dataset.DataSetRow",
{
	extend : xlsys.java.io.Serializable,
	implement : [xlsys.core.dataset.IDataSetRow],
	
	statics :
	{
		STATUS_COMMON : 1,
		STATUS_VIRTUAL : 2
	},
	
	construct: function()
	{
		this.cells = new xlsys.java.util.ArrayList();
		this.setCellClassName("xlsys.core.dataset.DataSetCell");
		this.setStatus(this.self(arguments).STATUS_COMMON);
		this.properties = null;
	},
	
	members :
	{
		cells : null,
		cellClassName : null,
		status : null,
		oldStatus : null,
		properties : null,
		
		getSerializableMembers : function()
		{
			var arr = new Array();
			arr.push("cells");
			return arr;
		},
		
		setCellClassName : function(cellClassName)
		{
			this.cellClassName = cellClassName;
		},
		
		getCellClassName : function()
		{
			return this.cellClassName;
		},
		
		addCell : function(cell)
		{
			this.cells.add(cell);
		},

		removeCell : function(cellIndex)
		{
			this.cells.removeAt(cellIndex);
		},

		getCell : function(index)
		{
			return this.cells.get(index);
		},

		setCell : function(index, cell)
		{
			this.cells.set(index, cell);
		},

		getCells : function()
		{
			return this.cells;
		},

		setCells : function(cells)
		{
			this.cells = cells;
		},

		getStatus : function()
		{
			return this.status;
		},
		
		getOldStatus : function()
		{
			return this.oldStatus;
		},

		setStatus : function(status)
		{
			this.oldStatus = this.status;
			this.status = status;
		},

		getProperties : function()
		{
			return this.properties;
		},

		putProperty : function(key, value)
		{
			if(this.properties==null) this.properties = new xlsys.java.util.HashMap();
			this.properties.put(key, value);
		},

		containsProperty : function(key)
		{
			var contains = false;
			if(this.properties!=null) contains = this.properties.containsKey(key);
			return contains;
		},

		removeProperty : function(key)
		{
			if(this.properties!=null) this.properties.remove(key);
		},

		getProperty : function(key)
		{
			var value = null;
			if(this.properties!=null) value = this.properties.get(key);
			return value;
		},

		cloneOf : function(anotherRow)
		{
			this.cells.clear();
			for(var i=0;i<anotherRow.cells.size();++i)
			{
				var anotherCell = anotherRow.cells.get(i);
				var cell = xlsys.core.util.ReflectUtil.newInstance(this.getCellClassName());
				cell.cloneOf(anotherCell);
				this.cells.add(cell);
			}
			this.setStatus(anotherRow.getStatus());
		}
	}
});