import "./DataSet";
import "../ISortable";
import "../ISwapable";
import "../../java/util/LinkedHashMap";
import "./IStorableDataSet";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.dataset.SortableDataSet",
{
	extend : xlsys.core.dataset.DataSet,
	implement : [xlsys.core.ISortable, xlsys.core.ISwapable],
	
	statics : 
	{
		_sort : function(dataSet, sortColMap)
		{
			var sortColIdxMap = new xlsys.java.util.LinkedHashMap();
			var sortColKeys = sortColMap.keys();
			for(var i in sortColKeys)
			{
				var sortColName = sortColKeys[i];
				var colIdx = dataSet.getColAt(sortColName);
				if(colIdx==-1) continue;
				var sortType = sortColMap.get(sortColName);
				sortColIdxMap.put(colIdx, sortType);
			}
			var rowArray = dataSet._getFactRows().toArray();
			rowArray.sort(qx.Bootstrap.bind(this._sortRow, this, dataSet, sortColIdxMap));
			// rebuilt rows
			var curRow = dataSet.getRow();
			let isSds = qx.Class.implementsInterface(dataSet, xlsys.core.dataset.IStorableDataSet);
			dataSet.setEventLock(true);
			if(isSds) dataSet.setLockChange(true);
			dataSet.clearRows();
			for(var i in rowArray)
			{
				dataSet.addRow(rowArray[i]);
			}
			// redirect row cursor
			dataSet.gotoRow(curRow);
			dataSet.setEventLock(false);
			if(isSds) dataSet.setLockChange(false);
		},
		
		_sortRow : function(dataSet, sortColIdxMap, row1, row2)
		{
			var result = 0;
			var toSortColIndices = sortColIdxMap.keys();
			for(var i in toSortColIndices)
			{
				var colIdx = toSortColIndices[i];
				var sortType = sortColIdxMap.get(colIdx);
				var column = dataSet.getColumn(colIdx);
				var content = row1.getCell(colIdx).getContent();
				var oContent = row2.getCell(colIdx).getContent();
				if(content==null&&oContent==null) result = 0;
				else if(content!=null&&oContent==null) result = (sortType==xlsys.core.ISortable.ASC?1:-1);
				else if(content==null&&oContent!=null) result = (sortType==xlsys.core.ISortable.ASC?-1:1);
				else
				{
					if(column.getJavaClass()=="java.lang.String") result = content.localeCompare(oContent);
					else if(column.getJavaClass()=="java.math.BigDecimal")
					{
						if(content<oContent) result = -1;
						else if(content>oContent) result = 1;
						else result = 0;
					}
					else if(column.getJavaClass()=="java.util.Date")
					{
						var time1 = content.getTime();
						var time2 = oContent.getTime();
						if(time1<time2) result = -1;
						else if(time1>time2) result = 1;
						else result = 0;
					}
					else if(column.getJavaClass()=="[B")
					{
						var byteStr1 = content.toString();
						var byteStr2 = oContent.toString();
						result = byteStr1.localeCompare(byteStr2);
					}
					else
					{
						if(content<oContent) result = -1;
						else if(content>oContent) result = 1;
						else result = 0;
					}
					result = (sortType==xlsys.core.ISortable.ASC?result:result*-1);
				}
				if(result!=0) break;
			}
			return result;
		}
	},
	
	construct : function()
	{
		this.base(arguments);
		this.sortColMap = new xlsys.java.util.LinkedHashMap();
	},
	
	members : 
	{
		sortColMap : null,
		
		getSerializableMembers : function()
		{
			var arr = this.base(arguments);
			arr.push("sortColMap");
			return arr;
		},
		
		setSortCols : function(colNames, sortTypes)
		{
			this.sortColMap.clear();
			for(var i=0;i<colNames.length;++i)
			{
				this.sortColMap.put(colNames[i], sortTypes[i]);
			}
		},
		
		setSortColMap : function(sortColMap)
		{
			this.sortColMap.clear();
			this.sortColMap.putAll(sortColMap);
		},
		
		sort : function()
		{
			this.self(arguments)._sort(this, this.sortColMap);
		},
		
		swap : function(row1, row2)
		{
			if(row1==row2) return;
			var i = this.getRowAt(row1);
			var j = this.getRowAt(row2);
			var min = i<j?i:j;
			var max = i<j?j:i;
			this.setEventLock(true);
			// 先删除大的行
			var maxRow = this.removeRow(max);
			// 再删除小的行
			var minRow = this.removeRow(min);
			// 先插入大行到小的序号中
			this.addRow(min, maxRow);
			// 再插入小行到大的序号中
			this.addRow(max, minRow);
			this.setEventLock(false);
		}
	}
});