import "./DataSet";
import "./ITreeDataSet";
import "../../java/util/LinkedHashMap";
import "../ISortable";
import "./SortableDataSet";
import "../../java/util/ArrayList";
import "../../java/util/HashSet";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.dataset.TreeDataSet",
{
	extend : xlsys.core.dataset.DataSet,
	implement : [xlsys.core.dataset.ITreeDataSet],

	statics : 
	{
		_setTreeColumn : function(treeDataSet, colAt)
		{
			var sortColMap = new xlsys.java.util.LinkedHashMap();
			sortColMap.put(treeDataSet.getColName(colAt), xlsys.core.ISortable.ASC);
			xlsys.core.dataset.SortableDataSet._sort(treeDataSet, sortColMap);
			return true;
		},
		
		_getRootRows : function(treeDataSet)
		{
			var rootRows = new xlsys.java.util.ArrayList();
			var childrenSet = new xlsys.java.util.HashSet();
			var rowArr = treeDataSet._getFactRows().toArray();
			for(var i in rowArr)
			{
				var row = rowArr[i];
				if(!childrenSet.contains(row))
				{
					rootRows.add(row);
					var children = treeDataSet.getAllChildren(row);
					if(children!=null) childrenSet.addAll(children);
				}
			}
			return rootRows;
		},
		
		_getChildren : function(treeDataSet, rowAt)
		{
			var children = new xlsys.java.util.ArrayList();
			var treeColAt = treeDataSet.getTreeColAt();
			if(treeColAt<0) return children;
			var rows = treeDataSet._getFactRows();
			var row = rows.get(rowAt);
			var rowStr = "" + row.getCell(treeColAt).getContent();
			var lastStr = null;
			for(var i=rowAt+1;i<rows.size();++i)
			{
				var otherRow = rows.get(i);
				var otherRowStr = "" + otherRow.getCell(treeColAt).getContent();
				if((lastStr==null || (lastStr!=null&&!qx.lang.String.startsWith(otherRowStr, lastStr))) && qx.lang.String.startsWith(otherRowStr, rowStr)&&otherRowStr.length>rowStr.length)
				{
					children.add(otherRow);
					lastStr = otherRowStr;
				}
				else if(otherRowStr.length<=rowStr.length) break;
			}
			return children;
		},
		
		_getAllChildren : function(treeDataSet, rowAt)
		{
			var children = new xlsys.java.util.ArrayList();
			var treeColAt = treeDataSet.getTreeColAt();
			if(treeColAt<0) return children;
			var rows = treeDataSet._getFactRows();
			var row = rows.get(rowAt);
			var rowStr = "" + row.getCell(treeColAt).getContent();
			for(var i=rowAt+1;i<rows.size();++i)
			{
				var otherRow = rows.get(i);
				var otherRowStr = "" + otherRow.getCell(treeColAt).getContent();
				if(qx.lang.String.startsWith(otherRowStr, rowStr)&&otherRowStr.length>rowStr.length) children.add(otherRow);
				else break;
			}
			return children;
		},
		
		_getParentRow : function(treeDataSet, rowAt)
		{
			var parentRow = null;
			var treeColAt = treeDataSet.getTreeColAt();
			if(treeColAt<0) return parentRow;
			var rows = treeDataSet._getFactRows();
			var row = rows.get(rowAt);
			var rowStr = "" + row.getCell(treeColAt).getContent();
			for(var i=rowAt-1;i>=0;--i)
			{
				var otherRow = rows.get(i);
				var otherRowStr = "" + otherRow.getCell(treeColAt).getContent();
				if(qx.lang.String.startsWith(rowStr, otherRowStr)&&otherRowStr.length<rowStr.length)
				{
					parentRow = otherRow;
					break;
				}
			}
			return parentRow;
		},
		
		_getAllParent : function(treeDataSet, rowAt)
		{
			var parentList = new xlsys.java.util.ArrayList();
			var treeColAt = treeDataSet.getTreeColAt();
			if(treeColAt<0) return parentList;
			var rows = treeDataSet._getFactRows();
			var row = rows.get(rowAt);
			var rowStr = "" + row.getCell(treeColAt).getContent();
			for(var i=rowAt-1;i>=0;--i)
			{
				var otherRow = rows.get(i);
				var otherRowStr = "" + otherRow.getCell(treeColAt).getContent();
				if(qx.lang.String.startsWith(rowStr, otherRowStr)&&otherRowStr.length<rowStr.length) parentList.add(otherRow);
			}
			return parentList;
		}
	},
	
	construct : function()
	{
		this.base(arguments);
	},
	
	members : 
	{
		treeColName : null,
		
		setTreeColumn : function(colNameOrColAt)
		{
			var treeColName = null;
			if(qx.lang.Type.isNumber(colNameOrColAt)) treeColName = this.getColName(colNameOrColAt);
			else treeColName = colNameOrColAt;
			this.treeColName = treeColName;
			var colAt = this.getColAt(treeColName);
			this.checkColRegion(colAt);
			this.self(arguments)._setTreeColumn(this, colAt);
		},
		
		sort : function()
		{
			if(this.treeColName!=null) this.setTreeColumn(this.treeColName);
		},
		
		getTreeColumn : function()
		{
			return this.treeColName;
		},
		
		getTreeColAt : function()
		{
			return this.getColAt(this.treeColName);
		},
		
		getRootRows : function()
		{
			return this.self(arguments)._getRootRows(this);
		},

		getChildren : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			return this.self(arguments)._getChildren(this, rowAt);
		},

		getAllChildren : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			return this.self(arguments)._getAllChildren(this, rowAt);
		},

		getParentRow : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			return this.self(arguments)._getParentRow(this, rowAt);
		},

		getAllParent : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			return this.self(arguments)._getAllParent(this, rowAt);
		},

		removeRow : function(rowOrRowAt, removeChildren)
		{
			if(removeChildren==null) removeChildren = false;
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowAt(rowOrRowAt);
			this.checkRowRegion(rowAt);
			var removedRows = new xlsys.java.util.ArrayList();
			if(removeChildren)
			{
				// 删除子行
				var children = this.getAllChildren(rowAt);
				removedRows.addAll(children);
				for(var i=children.size()-1;i>=0;--i) this.base(arguments, children.get(i));
			}
			removedRows.add(this.base(arguments, rowAt));
			if(arguments.length==1)
			{
				if(removedRows.isEmpty()) return null;
				else return removedRows.get(0);
			}
			return removedRows;
		},
		
		asyncRemoveRow : async function(rowOrRowAt, removeChildren)
		{
			return this.removeRow(rowOrRowAt, removeChildren);
		}
	}
});
