import "./IEnvironment";
import "../../java/util/HashMap";
import "../util/ReflectUtil";
import "../util/ObjectUtil";
import "../../java/util/Collection";
import "../../java/util/ArrayList";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.env.Environment",
{
	extend : qx.core.Object,
	implement : [xlsys.core.env.IEnvironment],
	
	statics : 
	{
		DEFAULT_ENV_CLASS_NAME : "xlsys.core.env.Environment",
		DEFAULT_CFG_LOADER_CLASS : null,
		__envMap : null,
		
		setDefaultEnvClassName : function(className)
		{
			if(className==null) throw new Error("Class name can not be null");
			this.DEFAULT_ENV_CLASS_NAME = className;
		},
		
		getDefaultEnvClassName : function()
		{
			return this.DEFAULT_ENV_CLASS_NAME;
		},
		
		setDefaultConfigLoaderClassName : function(className)
		{
			if(className==null) throw new Error("Class name can not be null");
			this.DEFAULT_CFG_LOADER_CLASS = className;
		},
		
		getDefaultConfigLoaderClassName : function()
		{
			return this.DEFAULT_CFG_LOADER_CLASS;
		},
		
		/**
		 * getInstance();
		 * getInstance(className);
		 */
		getInstance : function(className)
		{
			if(className==null) className = this.getDefaultEnvClassName();
			if(this.__envMap==null) this.__envMap = new xlsys.java.util.HashMap();
			var env = this.__envMap.get(className);
			if(env==null)
			{
				env = xlsys.core.util.ReflectUtil.newInstance(className, null, false);
				this.__envMap.put(className, env);
				var configLoaderClassName = this.getDefaultConfigLoaderClassName();
				if(configLoaderClassName!=null)
				{
					var configLoader = xlsys.core.util.ReflectUtil.newInstance(configLoaderClassName);
					configLoader.loadConfig(env);
				}
			}
			return env;
		}
	},
	
	construct : function()
	{
		this.base(arguments);
		this.__env = new xlsys.java.util.HashMap();
	},
	
	members : 
	{
		__env : null,

		/**
		 * getString(key)
		 * getString(key, defaultValue)
		 */
		getString : function(key, defaultValue)
		{
			var value = this.__env.get(key);
			if(value==null) value = defaultValue;
			return xlsys.core.util.ObjectUtil.objectToString(value);
		},
		
		setString : function(key, value)
		{
			this.__env.put(key, value);
		},
		
		getEnvironment : function(key)
		{
			var value = this.__env.get(key);
			if(value==null) return null;
			else if(qx.Class.implementsInterface(value, xlsys.java.util.Collection)) return this.__env.get(key).toArray()[0];
			else if(qx.lang.Type.isString(value)) throw new Error("String cannot be cast to xlsys.core.env.IEnvironment");
			return this.__env.get(key);
		},
		
		setEnvironment : function(key, value)
		{
			this.__env.put(key, value);
		},
		
		containsKey : function(key)
		{
			return this.__env.containsKey(key);
		},
		
		getValueClass : function(key)
		{
			var value = this.__env.get(key);
			if(value==null) return null;
			if(qx.lang.Type.isString(value)) return qx.type.BaseString;
			return xlsys.core.util.ObjectUtil.getClass(value);
		},
		
		getEnvironments : function(key)
		{
			var value = this.__env.get(key);
			if(value==null) return null;
			else if(qx.Class.implementsInterface(value, xlsys.core.env.IEnvironment))
			{
				var collection = new xlsys.java.util.ArrayList();
				collection.add(value);
				return collection;
			}
			else if(qx.lang.Type.isString(value)) throw new Error("String cannot be cast to java.util.Collection");
			return this.__env.get(key);
		},
		
		setEnvironments : function(key, environments)
		{
			this.__env.put(key, environments);
		},
		
		remove : function(key)
		{
			this.__env.remove(key);
		},
		
		keySet : function()
		{
			return this.__env.keySet();
		},
		
		addAll : function(another)
		{
			var keys = another.keySet().toArray();
			for(var i in keys) 
			{
				var key = keys[i];
				var valueClass = another.getValueClass(key);
				if(qx.Class.isSubClassOf(valueClass, qx.type.BaseString)) this.setString(key, another.getString(key));
				else if(qx.Class.hasInterface(valueClass, xlsys.java.util.Collection)) this.setEnvironments(key, another.getEnvironments(key));
				else if(qx.Class.hasInterface(valueClass, xlsys.core.env.IEnvironment)) this.setEnvironment(key, another.getEnvironment(key));
			}
		}
	}
});
