var qx = require("xlsys-qx");

qx.Mixin.define("xlsys.core.event.MAsyncListener", 
{
	members : 
	{
		// <type, lsnArr<obj>>
		_asyncLsnMap : null,
		// <id, obj>
		_asyncLsnIdMap : null,
		_asyncLsnIdCounter : 0,
		
		_initAsyncListener : function()
		{
			if(this._asyncLsnMap==null) this._asyncLsnMap = new Object();
			if(this._asyncLsnIdMap==null) this._asyncLsnIdMap = new Object();
		},
		
		addAsyncListener : function(type, listener, self)
		{
			return this._addAsyncListener(type, listener, self, false);
		},
		
		_addAsyncListener : function(type, listener, self, once)
		{
			this._initAsyncListener();
			if(type==null||listener==null) return null;
			if(!this._asyncLsnMap[type]) this._asyncLsnMap[type] = new Array();
			let tempArr = this._asyncLsnMap[type];
			let obj = new Object();
			obj.type = type;
			obj.listener = listener;
			obj.self = self;
			obj.once = once;
			let id = this._asyncLsnIdCounter++;
			obj.id = id;
			tempArr.push(obj);
			this._asyncLsnIdMap[id] = obj;
			return id;
		},
		
		addAsyncListenerOnce : function(type, listener, self)
		{
			return this._addAsyncListener(type, listener, self, true);
		},
		
		removeAsyncListenerById : function(id)
		{
			this._initAsyncListener();
			if(id==null) return;
			let obj = this._asyncLsnIdMap[id];
			if(obj==null) return;
			let type = obj.type;
			let tempArr = this._asyncLsnMap[type];
			if(tempArr==null) return;
			qx.lang.Array.remove(tempArr, obj);
		},
		
		fireAsyncEvent : async function(type, byOrder)
		{
			if(byOrder==null) byOrder = false;
			this._initAsyncListener();
			let tempArr = this._asyncLsnMap[type];
			if(tempArr==null) return;
			if(byOrder)
			{
				let event = new qx.event.type.Event();
				event.setTarget(this);
				for(let i=0;i<tempArr.length;++i)
				{
					let obj = tempArr[i];
					await this._runAsyncEvent(obj.id, obj.listener, obj.self, obj.once, event);
				}
			}
			else
			{
				let allPromises = new Array();
				let event = new qx.event.type.Event();
				event.setTarget(this);
				for(let i=0;i<tempArr.length;++i)
				{
					let obj = tempArr[i];
					allPromises.push(this._runAsyncEvent(obj.id, obj.listener, obj.self, obj.once, event));
				}
				await Promise.all(allPromises);
			}
		},
		
		_runAsyncEvent : async function(id, listener, self, once, event)
		{
			await listener.call(self, event);
			if(once) this.removeAsyncListenerById(id);
		},
		
		fireAsyncDataEvent : async function(type, data, byOrder)
		{
			if(byOrder==null) byOrder = false;
			this._initAsyncListener();
			let tempArr = this._asyncLsnMap[type];
			if(tempArr==null) return;
			if(byOrder)
			{
				let event = new qx.event.type.Data();
				event.init(data);
				event.setTarget(this);
				for(let i=0;i<tempArr.length;++i)
				{
					let obj = tempArr[i];
					await this._runAsyncEvent(obj.id, obj.listener, obj.self, obj.once, event);
				}
			}
			else
			{
				let allPromises = new Array();
				let event = new qx.event.type.Data();
				event.init(data);
				event.setTarget(this);
				for(let i=0;i<tempArr.length;++i)
				{
					let obj = tempArr[i];
					allPromises.push(this._runAsyncEvent(obj.id, obj.listener, obj.self, obj.once, event));
				}
				await Promise.all(allPromises);
			}
		}
	}
});