import "./ILog";
import "../env/Environment";
import "../util/ObjectUtil";
import "../Core";
import "../util/ReflectUtil";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.log.Log", 
{
	type : "abstract",
	extend : qx.core.Object,
	implement : [xlsys.core.log.ILog],
	
	statics : 
	{
		log : null,
		
		getInstance : function()
		{
			if(this.log==null)
			{
				var env = xlsys.core.env.Environment.getInstance();
				var logLevel = xlsys.core.util.ObjectUtil.objectToInt(env.getString(xlsys.core.Core.ENV_LOG_LEVEL, ""+xlsys.core.log.ILog.LOG_LEVEL_DEBUG));
				var logClass = env.getString(xlsys.core.Core.ENV_LOG_CLASS, "xlsys.core.log.ConsoleLog");
				this.log = xlsys.core.util.ReflectUtil.newInstance(logClass);
				this.log.setLogLevel(logLevel);
			}
			return this.log;
		}
	},
	
	construct : function()
	{
		this.base(arguments);
		this.logLevel = xlsys.core.log.ILog.LOG_LEVEL_DEBUG;
	},
	
	members : 
	{
		logLevel : null,
		
		/**
		 * print(level, content);
		 * print(content);
		 */
		print : function()
		{
			var level = null;
			var content = null;
			if(arguments.length==1)
			{
				level = this.logLevel;
				content = arguments[0];
			}
			else if(arguments.length==2)
			{
				level = arguments[0];
				content = arguments[1];
			}
			if(level<=this.logLevel) return this.write(level, content);
			return 0;
		},

		printDebug : function(content)
		{
			return this.print(xlsys.core.log.ILog.LOG_LEVEL_DEBUG, content);
		},

		printInfo : function(content)
		{
			return this.print(xlsys.core.log.ILog.LOG_LEVEL_INFO, content);
		},
		
		printWarn : function(content)
		{
			return this.print(xlsys.core.log.ILog.LOG_LEVEL_WARN, content);
		},

		printError : function(content)
		{
			return this.print(xlsys.core.log.ILog.LOG_LEVEL_ERROR, content);
		},

		printFatal : function(content)
		{
			return this.print(xlsys.core.log.ILog.LOG_LEVEL_FATAL, content);
		},
		
		write : function(level, content)
		{
			throw new Error("Abstract method call");
		},
		
		/**
		 * println(level, content);
		 * println(content);
		 */
		println : function()
		{
			var level = null;
			var content = null;
			if(arguments.length==1)
			{
				level = this.logLevel;
				content = arguments[0];
			}
			else if(arguments.length==2)
			{
				level = arguments[0];
				content = arguments[1];
			}
			if(level<=this.logLevel) return this.writeln(level, content);
			return 0;
		},
		
		printlnDebug : function(content)
		{
			return this.println(xlsys.core.log.ILog.LOG_LEVEL_DEBUG, content);
		},
		
		printlnInfo : function(content)
		{
			return this.println(xlsys.core.log.ILog.LOG_LEVEL_INFO, content);
		},
		
		printlnWarn : function(content)
		{
			return this.println(xlsys.core.log.ILog.LOG_LEVEL_WARN, content);
		},
		
		printlnError : function(content)
		{
			return this.println(xlsys.core.log.ILog.LOG_LEVEL_ERROR, content);
		},
		
		printlnFatal : function(content)
		{
			return this.println(xlsys.core.log.ILog.LOG_LEVEL_FATAL, content);
		},
		
		writeln : function(level, content)
		{
			var ret = this.write(level, content);
			if(ret>=0) ret += this.write(level, "\n");
			return ret;
		},

		setLogLevel : function(logLevel)
		{
			this.logLevel = logLevel;
		},

		getLogLevel : function()
		{
			return this.logLevel;
		}
	}
});
