import "./IModel";
import "../../java/io/Serializable";
import "../../java/util/ArrayList";
import "../../java/util/HashMap";
var qx = require("xlsys-qx");

/**
 * 树形结构模型
 */
qx.Class.define("xlsys.core.model.TreeModel",
{
	extend : xlsys.java.io.Serializable,
	implement : [xlsys.core.model.IModel],
	
	statics : 
	{
		STYLE_NORMAL : 0,
		STYLE_UNIQUE : 1
	},
	
	construct : function(data, style)
	{
		this.data = data;
		if(style==null) style = this.self(arguments).STYLE_NORMAL;
		this.style = style;
	},
	
	members : 
	{
		style : null,
		id : null,
		name : null,
		data : null,
		parent : null,
		children : null,
		properties : null,
		
		getData : function()
		{
			return this.data;
		},
		
		getId : function()
		{
			return this.id;
		},

		setId : function(id)
		{
			this.id = id;
		},

		getName : function()
		{
			return this.name;
		},

		setName : function(name)
		{
			this.name = name;
		},

		setParentData : function(parentData)
		{
			return this.setParent(new xlsys.core.model.TreeModel(parentData));
		},
		
		setParent : function(parent)
		{
			this.parent = parent;
			if(parent!=null)
			{
				if(this.style==this.self(arguments).STYLE_NORMAL||(this.style==this.self(arguments).STYLE_UNIQUE&&!parent.containsChild(this)))
				{
					parent.addChild(this);
				}
			}
			return parent;
		},
		
		addChildData : function(childData)
		{
			return this.addChild(new xlsys.core.model.TreeModel(childData));
		},
		
		addChild : function(child)
		{
			if(this.children==null) this.children = new xlsys.java.util.ArrayList();
			this.children.add(child);
			child.parent = this;
			return child;
		},
		
		containsChildData : function(childData)
		{
			return this.containsChild(new xlsys.core.model.TreeModel(childData));
		},
		
		containsChild : function(child)
		{
			if(this.children!=null) return this.children.contains(child);
			return false;
		},
		
		getParentData : function()
		{
			if(this.parent!=null) return this.parent.data;
			return null;
		},
		
		getParent : function()
		{
			return this.parent;
		},
		
		getChildData : function(index)
		{
			if(this.children!=null) return this.children.get(index).data;
			return null;
		},
		
		getChild : function(index)
		{
			if(this.children!=null) return this.children.get(index);
			return null;
		},
		
		getChildren : function()
		{
			if(this.children!=null) return this.children;
			else return new xlsys.java.util.ArrayList();
		},
		
		putProperty : function(key, value)
		{
			if(this.properties==null) this.properties = new xlsys.java.util.HashMap();
			this.properties.put(key, value);
		},
		
		getProperty : function(key)
		{
			if(this.properties!=null) return this.properties.get(key);
			return null;
		},
		
		containsProperty : function(key)
		{
			if(this.properties!=null) return this.properties.containsKey(key);
			return false;
		},
		
		getSerializableMembers : function()
		{
			var arr = new Array();
			arr.push("style");
			arr.push("id");
			arr.push("name");
			arr.push("data");
			arr.push("parent");
			arr.push("children");
			arr.push("properties");
			return arr;
		}
	}
});