import "../../java/util/HashMap";
import "../../java/util/HashSet";
import "../service/TaskEvent";
import "../../java/util/ArrayList";
import "../log/Log";
var qx = require("xlsys-qx");

qx.Class.define("xlsys.core.service.ServiceManager",
{
	type : "singleton",
	extend : qx.core.Object,

	construct : function()
	{
		this.serviceMap = new xlsys.java.util.HashMap();
	},
	
	members : 
	{
		serviceMap : null,
		
		registerService : function(taskType, service)
		{
			var serviceSet = this.serviceMap.get(taskType);
			if(serviceSet==null)
			{
				serviceSet = new xlsys.java.util.HashSet();
				this.serviceMap.put(taskType, serviceSet);
			}
			serviceSet.add(service);
			service.mount();
		},
		
		unregisterService : function(taskType, service)
		{
			var serviceSet = this.serviceMap.get(taskType);
			if(serviceSet!=null)
			{
				serviceSet.remove(service);
				service.unmount();
			}
		},
	
		canHandlerTask : function(task)
		{
			var can = false;
			var event = new xlsys.core.service.TaskEvent(this);
			event.task = task;
			var serviceSet = this.serviceMap.get(task.getTaskType());
			if(serviceSet!=null)
			{
				var services = serviceSet.toArray();
				for(var i in services)
				{
					var service = services[i];
					if(service.canHandlerTask(event))
					{
						can = true;
						break;
					}
				}
			}
			return can;
		},
	
		containsService : function(taskType, service)
		{
			var contains = false;
			var serviceSet = this.serviceMap.get(taskType);
			if(serviceSet!=null) contains = serviceSet.contains(service);
			return contains;
		},
	
		getRegistedServices : function(taskType)
		{
			var serviceList = new xlsys.java.util.ArrayList();
			var serviceSet = this.serviceMap.get(taskType);
			if(serviceSet!=null) serviceList.addAll(serviceSet);
			return serviceList;
		},
	
		addTask : async function(publisher, task)
		{
			return await this._executeTask(task, publisher);
		},
		
		_executeTask : async function(task, publisher)
		{
			var taskType = task.getTaskType();
			var services = null;
			var serviceSet = this.serviceMap.get(taskType);
			if(serviceSet!=null) services = serviceSet.toArray();
			if(services==null) return;
			// 遍历所有注册了该任务类型的Service
			for(var i in services)
			{
				var service = services[i];
				try
				{
					// 询问是否能处理该任务
					var event = new xlsys.core.service.TaskEvent(this);
					event.service = service;
					event.task = task;
					event.publisher = publisher;
					if(service.canHandlerTask(event))
					{
						// 询问是否需要中断该任务的发布, 默认为不中断
						var interrupt = service.interruptPublish(event);
						if(interrupt==null) interrupt = false;
						// 处理任务
						await this._dealWithTask(event);
						// 如果任务不需要继续发布, 则中断执行
						if(interrupt) break;
					}
				}
				catch(e)
				{
					xlsys.core.log.Log.getInstance().printlnError(e);
				}
			}
		},
		
		_dealWithTask : async function(event)
		{
			// 执行publisher的开始任务前事件
			await event.publisher.beforeTaskRun(event);
			if(!event.doit) return;
			// 处理任务
			await event.service.handlerTask(event);
			// 执行publisher的任务结束后事件
			await event.publisher.afterTastRun(event);
			return;
		}
	}
});
