import "../../java/lang/StringBuilder";
import "../io/ByteDataInputStream";
import "../../java/util/HashMap";
import "../Core";
import "./StringUtil";
import "../../java/util/SimpleEntry";
import "./ObjectUtil";
import "../../java/util/LinkedHashMap";
import "../io/ByteDataOutputStream";
import "../../java/util/Entry";
import "../../java/util/Collection";
import "../../java/util/Map";
import "../../java/lang/Exception";
import "./StringUtil";
import "../dataset/IDataSetRow";
import "../../java/io/Serializable";
import "../dataset/DataSetCell";
var qx = require("xlsys-qx");

/**
 * io工具类
 */
qx.Class.define("xlsys.core.util.IOUtil",
{
	type : "static",
	
	statics :
	{
		__rRegExp : new RegExp("\r", "g"),
		__nRegExp : new RegExp("\n", "g"),
		__tRegExp : new RegExp("\t", "g"),
		__dqRegExp : new RegExp("\"", "g"),
		__escapeRRegExp : new RegExp("&#13;", "g"),
		__escapeNRegExp : new RegExp("&#10;", "g"),
		__escapeTRegExp : new RegExp("&#09;", "g"),
		__escapeDQRegExp : new RegExp("&#34;", "g"),
		
		getDQMStr : function(str)
		{
			return "\""+str+"\"";
		},

		createJSONAttr : function(attrName, attrValue)
		{
			var sb = new xlsys.java.lang.StringBuilder();
			sb.append("\"").append(attrName).append("\":");
			if(attrValue==null) sb.append("null");
			else if((typeof attrValue)=="string") sb.append("\"").append(attrValue).append("\"");
			else sb.append(attrValue);
			return sb.toString();
		},
		
		readInternalObject : function(bytes)
		{
			if(bytes==null||bytes.length==0) return null;
			var dis = new xlsys.core.io.ByteDataInputStream(bytes);
			var firstHashcode = 0;
			// 获取所有引用的对象
			var hasWriteObjMap = new xlsys.java.util.HashMap();
			// 读取总长度
			var totalLength = dis.readInt();
			// 读取所有内容
			var totalContent = new Array(totalLength);
			dis.readFully(totalContent, 0, totalLength);
			var tempDis = new xlsys.core.io.ByteDataInputStream(totalContent);
			while(tempDis.available()>0)
			{
				// 读取hashcode
				var hashcode = tempDis.readInt();
				if(firstHashcode==0) firstHashcode = hashcode;
				// 读取长度
				var length = tempDis.readInt();
				// 读取内容
				var content = new Array(length);
				tempDis.readFully(content, 0, length);
				hasWriteObjMap.put(hashcode, content);
			}
			// 生成第一个对象
			var rebuiltObjMap = new xlsys.java.util.HashMap();
			return this.rebuildInternalObject(firstHashcode, hasWriteObjMap, rebuiltObjMap);
		},
		
		rebuildInternalObject : function(hashcode, hasWriteObjMap, rebuiltObjMap)
		{
			var obj = null;
			if(rebuiltObjMap.containsKey(hashcode)) obj = rebuiltObjMap.get(hashcode);
			else
			{
				var content = hasWriteObjMap.get(hashcode);
				var dis = new xlsys.core.io.ByteDataInputStream(content);
				// 读取类型
				var type = dis.readByte();
				if(type==xlsys.core.Core.DATA_TYPE_NULL)
				{
						// Do Nothing
				}
				else if(type==xlsys.core.Core.DATA_TYPE_STR)
				{
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					obj = xlsys.core.util.StringUtil.bytesToString(bytes);
				}
				else if(type==xlsys.core.Core.DATA_TYPE_STR_ARRAY)
				{
					// 数组对象
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					obj = new Array();
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.push(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.core.Core.DATA_TYPE_SHORT)
				{
					obj = dis.readShort();
				}
				else if(type==xlsys.core.Core.DATA_TYPE_INT)
				{
					obj = dis.readInt();
				}
				else if(type==xlsys.core.Core.DATA_TYPE_LONG)
				{
					obj = dis.readLong();
				}
				else if(type==xlsys.core.Core.DATA_TYPE_CHAR)
				{
					obj = dis.readChar();
				}
				else if(type==xlsys.core.Core.DATA_TYPE_BOOL)
				{
					obj = dis.readByte()==1?true:false;
				}
				else if(type==xlsys.core.Core.DATA_TYPE_FLOAT)
				{
					throw new Error("Unsupported!")
				}
				else if(type==xlsys.core.Core.DATA_TYPE_DOUBLE)
				{
					throw new Error("Unsupported!")
				}
				else if(type==xlsys.core.Core.DATA_TYPE_BYTE)
				{
					obj = dis.readByte();
				}
				else if(type==xlsys.core.Core.DATA_TYPE_BYTE_ARRAY)
				{
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					obj = bytes;
				}
				else if(type==xlsys.core.Core.DATA_TYPE_BIGDECIMAL)
				{
					// 读取数字字符串的hashcode
					var numHashcode = dis.readInt();
					var numStr = this.rebuildInternalObject(numHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = Number(numStr);
				}
				else if(type==xlsys.core.Core.DATA_TYPE_DATE)
				{
					// 读取时间long
					var time = dis.readLong();
					obj = new Date(time);
					obj.setTime(time);
				}
				else if(type==xlsys.core.Core.DATA_TYPE_ENTRY)
				{
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					// 读取key
					var tempHashcode = tempIn.readInt();
					var key = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取value
					tempHashcode = tempIn.readInt();
					var value = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = new xlsys.java.util.SimpleEntry(key, value);
				}
				else if(type==xlsys.core.Core.DATA_TYPE_ENTRY_ARRAY)
				{
					// 数组对象
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var obj = new Array();
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.push(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.core.Core.DATA_TYPE_OBJECT_ARRAY)
				{
					// 数组对象
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var obj = new Array();
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.push(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.core.Core.DATA_TYPE_COLLECTION)
				{
					// 集合
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.core.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.add(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.core.Core.DATA_TYPE_MAP)
				{
					// Map
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.core.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取key
						var tempHashcode = tempIn.readInt();
						var key = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
						// 读取value
						tempHashcode = tempIn.readInt();
						var value = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
						// 放入map
						obj.put(key, value);
					}
				}
				else if(type==xlsys.core.Core.DATA_TYPE_EXCEPTION)
				{
					// Exception
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.core.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					// 填充对象Field
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					// 读取虚拟field : msg
					// 读取名称(这里不使用)
					var fieldNameHashcode = tempIn.readInt();
					this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取内容
					var msgHashcode = tempIn.readInt();
					var message = this.rebuildInternalObject(msgHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取虚拟field : stackTrace
					// 读取名称(这里不使用)
					fieldNameHashcode = tempIn.readInt();
					this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取内容
					var stackTraceHashcode = tempIn.readInt();
					var stackTrace = this.rebuildInternalObject(stackTraceHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取虚拟field : cause
					// 读取名称(这里不使用)
					fieldNameHashcode = tempIn.readInt();
					this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取内容
					var causeHashcode = tempIn.readInt();
					var cause = this.rebuildInternalObject(causeHashcode, hasWriteObjMap, rebuiltObjMap);
					// 创建对象
					obj.msg = message;
					obj.stackTrace = stackTrace;
					obj.cause = cause;
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
				}
				else
				{
					// 为复杂对象类型
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.core.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					// 填充对象Field
					var tempIn = new xlsys.core.io.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取Field名称
						var fieldNameHashcode = tempIn.readInt();
						var fieldName = this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
						// 获取Field对象
						var tempHashcode = tempIn.readInt();
						var fieldObj = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
						// 设置field到obj对象
						obj[fieldName] = fieldObj;
					}
				}
				rebuiltObjMap.put(hashcode, obj);
			}
			return obj;
		},
		
		getInternalObjectBytes : function(obj)
		{
			// 获取参数的对象中包含的所有对象的字节数组Map,复杂类型均使用hashcode代表
			var hasWriteObjMap = new xlsys.java.util.LinkedHashMap();
			this.prepareInternalBytes(obj, hasWriteObjMap);
			// 将Map写成byte数组
			var baos = new xlsys.core.io.ByteDataOutputStream();
			var keys = hasWriteObjMap.keys();
			for(var i=0;i<keys.length;++i)
			{
				// 写入hashcode
				var hashcode = keys[i];
				baos.writeInt(hashcode);
				// 写入长度
				var content = hasWriteObjMap.get(hashcode);
				baos.writeInt(content.length);
				// 写入内容
				baos.write(content);
			}
			var out = new xlsys.core.io.ByteDataOutputStream();
			var temp = baos.toByteArray();
			// 写入总长度
			out.writeInt(temp.length);
			// 写入总内容
			out.write(temp);
			return out.toByteArray();
		},
		
		prepareInternalBytes : function(obj, hasWriteObjMap, dataType)
		{
			if(hasWriteObjMap.containsKey(xlsys.core.util.ObjectUtil.hashcode(obj))) return;
			// 先将当前obj放入,以免子属性递归时重复引用造成无限循环
			hasWriteObjMap.put(xlsys.core.util.ObjectUtil.hashcode(obj), null);
			var data = null;
			var out = new xlsys.core.io.ByteDataOutputStream();
			if(obj==null)
			{
				// 写入类型
				out.writeByte(xlsys.core.Core.DATA_TYPE_NULL);
			}
			else
			{
				if(dataType==null)
				{
					// 没有dataType, 则先根据数据类型来尝试获取dataType
					if(qx.lang.Type.isString(obj)) dataType = xlsys.core.Core.DATA_TYPE_STR;
					else if(qx.lang.Type.isArray(obj)) dataType = xlsys.core.Core.DATA_TYPE_OBJECT_ARRAY;
					else if(qx.lang.Type.isNumber(obj)) dataType = xlsys.core.Core.DATA_TYPE_BIGDECIMAL;
					else if(qx.lang.Type.isBoolean(obj)) dataType = xlsys.core.Core.DATA_TYPE_BOOL;
					else if(qx.lang.Type.isDate(obj)) dataType = xlsys.core.Core.DATA_TYPE_DATE;
					else if(qx.Class.implementsInterface(obj, xlsys.java.util.Entry)) dataType = xlsys.core.Core.DATA_TYPE_ENTRY;
					else if(qx.Class.implementsInterface(obj, xlsys.java.util.Collection)) dataType = xlsys.core.Core.DATA_TYPE_COLLECTION;
					else if(qx.Class.implementsInterface(obj, xlsys.java.util.Map)) dataType = xlsys.core.Core.DATA_TYPE_MAP;
					else if(obj instanceof xlsys.java.lang.Exception) dataType = xlsys.core.Core.DATA_TYPE_EXCEPTION;
					else dataType = xlsys.core.Core.DATA_TYPE_OBJECT;
				}
				if(dataType==xlsys.core.Core.DATA_TYPE_STR)
				{
					var bytes = xlsys.core.util.StringUtil.stringToBytes(obj);
					// 写入类型
					out.writeByte(xlsys.core.Core.DATA_TYPE_STR);
					// 写入长度
					out.writeInt(bytes.length);
					// 写入内容
					out.write(bytes);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_SHORT)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_SHORT);
					out.writeShort(obj);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_INT)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_INT);
					out.writeInt(obj);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_LONG)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_LONG);
					out.writeLong(obj);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_CHAR)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_CHAR);
					out.writeChar(obj);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_BOOL)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_BOOL);
					out.writeByte(obj?1:0);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_BYTE)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_BYTE);
					out.writeByte(obj);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_BYTE_ARRAY)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_BYTE_ARRAY);
					out.writeInt(obj.length);
					out.write(obj);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_BIGDECIMAL)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_BIGDECIMAL);
					var numStr = "" + obj;
					// 写入数字文本的hashcode
					out.writeInt(xlsys.core.util.ObjectUtil.hashcode(numStr));
					this.prepareInternalBytes(numStr, hasWriteObjMap);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_DATE)
				{
					out.writeByte(xlsys.core.Core.DATA_TYPE_DATE);
					// 写入时间long
					out.writeLong(obj.getTime());
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_ENTRY)
				{
					// 写入类型
					out.writeByte(xlsys.core.Core.DATA_TYPE_ENTRY);
					var tempOut = new xlsys.core.io.ByteDataOutputStream();
					// 写入Key
					var key = obj.getKey();
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(key));
					this.prepareInternalBytes(key, hasWriteObjMap);
					// 写入Value
					var value = obj.getValue();
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(value));
					this.prepareInternalBytes(value, hasWriteObjMap);
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_ENTRY_ARRAY||
						dataType==xlsys.core.Core.DATA_TYPE_OBJECT_ARRAY||
						dataType==xlsys.core.Core.DATA_TYPE_STR_ARRAY)
				{
					// 数组对象
					// 写入类型
					out.writeByte(dataType);
					var tempOut = new xlsys.core.io.ByteDataOutputStream();
					for(var i=0;i<obj.length;++i)
					{
						var temp = obj[i];
						// 写入元素的hashcode
						tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(temp));
						// 递归遍历元素
						this.prepareInternalBytes(temp, hasWriteObjMap);
					}
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_COLLECTION)
				{
					// 集合
					// 写入类型
					out.writeByte(xlsys.core.Core.DATA_TYPE_COLLECTION);
					// 写入名称
					var name = xlsys.core.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.core.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					var tempOut = new xlsys.core.io.ByteDataOutputStream();
					var collectionArr = obj.toArray();
					for(var i=0;i<collectionArr.length;++i)
					{
						var temp = collectionArr[i];
						// 写入元素的hashcode
						tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(temp));
						// 递归遍历元素
						this.prepareInternalBytes(temp, hasWriteObjMap);
					}
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_MAP)
				{
					// Map
					// 写入类型
					out.writeByte(xlsys.core.Core.DATA_TYPE_MAP);
					// 写入名称
					var name = xlsys.core.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.core.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					var tempOut = new xlsys.core.io.ByteDataOutputStream();
					var keys = obj.keys();
					for(var i=0;i<keys.length;++i)
					{
						var key = keys[i];
						var value = obj.get(key);
						// 写入Key
						tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(key));
						this.prepareInternalBytes(key, hasWriteObjMap);
						// 写入Value
						tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(value));
						this.prepareInternalBytes(value, hasWriteObjMap);
					}
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_EXCEPTION)
				{
					// 写入类型
					out.writeByte(xlsys.core.Core.DATA_TYPE_EXCEPTION);
					// 写入名称
					var name = xlsys.core.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.core.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					var tempOut = new xlsys.core.io.ByteDataOutputStream();
					// 插入虚拟field : msg
					var fieldName = "msg";
					var message = obj.getMessage();
					if(message==null) message = "";
					// 写入Field名称
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(fieldName));
					this.prepareInternalBytes(fieldName, hasWriteObjMap);
					// 写入Field的hashcode
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(message));
					this.prepareInternalBytes(message, hasWriteObjMap);
					// 插入虚拟field : stackTrace
					fieldName = "stackTrace";
					var stackTrace = obj.getStackTrace();
					// 写入Field名称
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(fieldName));
					this.prepareInternalBytes(fieldName, hasWriteObjMap);
					// 写入Field的hashcode
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(stackTrace));
					this.prepareInternalBytes(stackTrace, hasWriteObjMap, xlsys.core.Core.DATA_TYPE_STR_ARRAY);
					// 插入虚拟field : cause
					fieldName = "cause";
					var cause = obj.getCause();
					// 写入Field名称
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(fieldName));
					this.prepareInternalBytes(fieldName, hasWriteObjMap);
					// 写入Field的hashcode
					tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(cause));
					this.prepareInternalBytes(cause, hasWriteObjMap);
					// 写入长度
					var fieldsData = tempOut.toByteArray();
					out.writeInt(fieldsData.length);
					// 写入内容
					out.write(fieldsData);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_OBJECT)
				{
					// 为复杂对象类型
					// 写入类型
					out.writeByte(xlsys.core.Core.DATA_TYPE_OBJECT);
					// 写入名称
					var name = xlsys.core.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.core.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					// 放入所有的member
					var tempOut = new xlsys.core.io.ByteDataOutputStream();
					var fields = obj.getSerializableMembers();
					for(var i=0;i<fields.length;++i)
					{
						var fieldName = fields[i];
						// 写入Field名称
						tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(fieldName));
						this.prepareInternalBytes(fieldName, hasWriteObjMap);
						// 写入Field的hashcode
						var fieldObj = obj[fieldName];
						tempOut.writeInt(xlsys.core.util.ObjectUtil.hashcode(fieldObj));
						// 获取Field的类型
						var fieldDataType = obj.getMemberDataType(fieldName);
						this.prepareInternalBytes(fieldObj, hasWriteObjMap, fieldDataType);
					}
					// 写入长度
					var fieldsData = tempOut.toByteArray();
					out.writeInt(fieldsData.length);
					// 写入内容
					out.write(fieldsData);
				}
			}
			data = out.toByteArray();
			hasWriteObjMap.put(xlsys.core.util.ObjectUtil.hashcode(obj), data);
		},
		
		bytesToHexStr : function(bytes)
		{
			var sb = new xlsys.java.lang.StringBuilder();
			for(var i=0;i<bytes.length;++i)
			{
				var temp = bytes[i].toString(16);
				if(temp.length<2) temp = "0" + temp;
				sb.append(temp);
			}
			return sb.toString();
		},
		
		hexStrToBytes : function(hexStr)
		{
			var bytes = new Array();
			for(var i=0;i<hexStr.length;i+=2)
			{
				bytes.push(parseInt(hexStr.substring(i, i+2), 16));
			}
			return bytes;
		},
		
		getCommonJSONFromObj : function(seriObj, dataType)
		{
			var sb = new xlsys.java.lang.StringBuilder();
			if(seriObj==null) sb.append("null");
			else if(qx.lang.Type.isArray(seriObj))
			{
				if((dataType!=null&&dataType==xlsys.core.Core.DATA_TYPE_BYTE_ARRAY)||
					(seriObj.length>0&&qx.lang.Type.isNumber(seriObj[0])&&qx.lang.Type.isNumber(seriObj[seriObj.length-1])))
				{
					sb.append("{");
					sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_BYTE_ARRAY)).append(",");
					sb.append(this.createJSONAttr("value", this.bytesToHexStr(seriObj)));
					sb.append("}");
				}
				else
				{
					sb.append("[");
					for(var i=0;i<seriObj.length;++i)
					{
						sb.append(this.getCommonJSONFromObj(seriObj[i]));
						if(i!=seriObj.length-1) sb.append(",");
					}
					sb.append("]");
				}
			}
			else if(qx.lang.Type.isBoolean(seriObj)) sb.append(seriObj);
			else if(qx.lang.Type.isDate(seriObj))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_DATE)).append(",");
				sb.append(this.createJSONAttr("value", seriObj.getTime()));
				sb.append("}");
			}
			else if(qx.lang.Type.isError(seriObj)||seriObj instanceof xlsys.java.lang.Exception)
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_EXCEPTION)).append(",");
				if(qx.lang.Type.isError(seriObj)) sb.append(this.createJSONAttr("value", seriObj.message));
				else sb.append(this.createJSONAttr("value", seriObj.getMessage()));
				sb.append("}");
			}
			else if(qx.lang.Type.isNumber(seriObj)) sb.append(seriObj);
			else if(qx.lang.Type.isString(seriObj))
			{
				if(seriObj.indexOf("\n")>=0||seriObj.indexOf("\r")>=0||seriObj.indexOf("\t")>=0||seriObj.indexOf("\"")>=0)
				{
					var temp = seriObj.replace(this.__nRegExp, "&#10;").replace(this.__rRegExp, "&#13;").replace(this.__tRegExp, "&#09;").replace(this.__dqRegExp, "&#34;");
					sb.append("\"").append(temp).append("\"");
				}
				else sb.append("\"").append(seriObj).append("\"");
			}
			else if(qx.Class.implementsInterface(seriObj, xlsys.java.util.Collection))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_COLLECTION)).append(",");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_CLASS, xlsys.core.util.ObjectUtil.className(seriObj))).append(",");
				sb.append("\"value\":").append("[");
				var tempArr = seriObj.toArray();
				for(var i=0;i<tempArr.length;++i)
				{
					sb.append(this.getCommonJSONFromObj(tempArr[i]));
					if(i!=tempArr.length-1) sb.append(",");
				}
				sb.append("]");
				sb.append("}");
			}
			else if(qx.Class.implementsInterface(seriObj, xlsys.java.util.Entry))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_ENTRY)).append(",");
				sb.append("\"key\":").append(this.getCommonJSONFromObj(seriObj.getKey()));
				sb.append("\"value\":").append(this.getCommonJSONFromObj(seriObj.getValue()));
				sb.append("}");
			}
			else if(qx.Class.implementsInterface(seriObj, xlsys.java.util.Map))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_MAP)).append(",");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_CLASS, xlsys.core.util.ObjectUtil.className(seriObj))).append(",");
				sb.append("\"value\":").append("[");
				var keyArr = seriObj.keys();
				for(var i=0;i<keyArr.length;++i)
				{
					sb.append(this.getCommonJSONFromObj(keyArr[i])).append(",");
					sb.append(this.getCommonJSONFromObj(seriObj.get(keyArr[i])));
					if(i!=keyArr.length-1) sb.append(",");
				}
				sb.append("]");
				sb.append("}");
			}
			else if(qx.Class.implementsInterface(seriObj, xlsys.core.dataset.IDataSetRow))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_DATA_SET_ROW)).append(",");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_CLASS, xlsys.core.util.ObjectUtil.className(seriObj))).append(",");
				sb.append("\"value\":").append("[");
				var cellArr = seriObj.getCells().toArray();
				for(var i=0;i<cellArr.length;++i)
				{
					sb.append(this.getCommonJSONFromObj(cellArr[i].getContent()));
					if(i!=cellArr.length-1) sb.append(",");
				}
				sb.append("]");
				sb.append("}");
			}
			else if(seriObj instanceof xlsys.java.io.Serializable)
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_DATA_TYPE, xlsys.core.Core.DATA_TYPE_OBJECT)).append(",");
				sb.append(this.createJSONAttr(xlsys.core.Core.JSON_OBJ_CLASS, xlsys.core.util.ObjectUtil.className(seriObj))).append(",");
				var fields = seriObj.getSerializableMembers();
				for(var i=0;i<fields.length;++i)
				{
					var fieldName = fields[i];
					var fieldDataType = seriObj.getMemberDataType(fieldName);
					sb.append("\"").append(fieldName).append("\"").append(":").append(this.getCommonJSONFromObj(seriObj[fieldName], fieldDataType));
					if(i!=fields.length-1) sb.append(",");
				}
				sb.append("}");
			}
			return sb.toString();
		},
		
		getObjFromCommonJSON : function(jsonStr)
		{
			var allJson = JSON.parse(jsonStr);
			return this._rebuildObjFromCommonJSON(allJson);
		},
		
		_rebuildObjFromCommonJSON : function(jsonObj)
		{
			var obj = null;
			if(jsonObj==null) obj = null;
			if(qx.lang.Type.isArray(jsonObj))
			{
				obj = new Array();
				for(var i=0;i<jsonObj.length;++i)
				{
					obj.push(this._rebuildObjFromCommonJSON(jsonObj[i]));
				}
			}
			else if(qx.lang.Type.isBoolean(jsonObj)) obj = jsonObj;
			else if(qx.lang.Type.isNumber(jsonObj)) obj = jsonObj;
			else if(qx.lang.Type.isString(jsonObj))
			{
				obj = jsonObj;
				// 还原\r,\n,\t,\"
				obj = obj.replace(this.__escapeNRegExp, "\n").replace(this.__escapeRRegExp, "\r").replace(this.__escapeTRegExp, "\t").replace(this.__escapeDQRegExp, "\"");
			}
			else if(qx.lang.Type.isObject(jsonObj))
			{
				// 获取类型
				var dataType = jsonObj[xlsys.core.Core.JSON_OBJ_DATA_TYPE];
				var className = jsonObj[xlsys.core.Core.JSON_OBJ_CLASS];
				if(dataType==xlsys.core.Core.DATA_TYPE_DATE)
				{
					obj = new Date();
					obj.setTime(jsonObj["value"]);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_EXCEPTION)
				{
					obj = new xlsys.java.lang.Exception(jsonObj["value"]);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_COLLECTION)
				{
					obj = xlsys.core.util.ObjectUtil.tryInitObj(className);
					if(obj==null) throw new Error("Unsupported");
					var contentJsonArr = jsonObj["value"];
					for(var i=0;i<contentJsonArr.length;++i)
					{
						obj.add(this._rebuildObjFromCommonJSON(contentJsonArr[i]));
					}
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_ENTRY)
				{
					var key = this._rebuildObjFromCommonJSON(jsonObj["key"]);
					var value = this._rebuildObjFromCommonJSON(jsonObj["value"]);
					obj = new xlsys.java.util.SimpleEntry(key, value);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_MAP)
				{
					obj = xlsys.core.util.ObjectUtil.tryInitObj(className);
					if(obj==null) throw new Error("Unsupported");
					var contentJsonArr = jsonObj["value"];
					for(var i=0;i<contentJsonArr.length;i+=2)
					{
						var key = this._rebuildObjFromCommonJSON(contentJsonArr[i]);
						var value = this._rebuildObjFromCommonJSON(contentJsonArr[i+1]);
						obj.put(key, value);
					}
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_BYTE_ARRAY)
				{
					var hexStr = jsonObj["value"];
					obj = this.hexStrToBytes(hexStr);
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_DATA_SET_ROW)
				{
					var row = xlsys.core.util.ObjectUtil.tryInitObj(className);
					var contentJsonArr = jsonObj["value"];
					for(var i=0;i<contentJsonArr.length;++i)
					{
						var dsc = new xlsys.core.dataset.DataSetCell();
						dsc.setContent(this._rebuildObjFromCommonJSON(contentJsonArr[i]));
						row.getCells().add(dsc);
					}
					obj = row;
				}
				else if(dataType==xlsys.core.Core.DATA_TYPE_OBJECT)
				{
					obj = xlsys.core.util.ObjectUtil.tryInitObj(className);
					if(obj==null) throw new Error("Unsupported");
					var fields = obj.getSerializableMembers();
					for(var i=0;i<fields.length;++i)
					{
						var fieldName = fields[i];
						obj[fieldName] = this._rebuildObjFromCommonJSON(jsonObj[fieldName]);
					}
				}
			}
			return obj;
		}
	}
});